package at.jku.ssw.fp.sect10_5;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import io.reactivex.Observable;
import io.reactivex.rxjavafx.schedulers.JavaFxScheduler;
import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.Scene;
import javafx.scene.control.ListView;
import javafx.scene.layout.StackPane;
import javafx.stage.Stage;

/**
 * Demo program to show observeOn with JavaFxScheduler.
 *
 * Start program with {@code mvn clean javafx:run} (see README.md).
 *
 * @author Herbert Praehofer
 *
 */
public class Demo4_WeatherData_ObserveOnFX extends Application {

  private static int N = 60;
  private static String addresss = "https://api.openweathermap.org/data/2.5/weather?appid=f0d81b7ec9a918a6c4e01fcba9fe537a&lat=48&lon=14";

  public static void main(String[] args) {
     launch(args);
  }

  @Override
  public void start(Stage primaryStage) throws Exception {

    ObservableList<String> items = FXCollections.observableArrayList();
    ListView<String> list = new ListView<>();
    list.setItems(items);

    URL url = new URL(addresss);
    Observable<WeatherData> weatherDataObservable =
       Observable
         .interval(0, 10, TimeUnit.SECONDS)
         .map(i -> getWeatherData(url))
         .map(json -> WeatherData.fromJSON(json))
         .publish()
         .refCount();

    weatherDataObservable
      .map(wData -> wData.temp)
      .map(f -> (f - 32.0) * 5.0 / 9.0)
      .observeOn(JavaFxScheduler.platform())
      .subscribe(temp -> {
        if (items.size() == N) items.remove(0);
        items.add(String.format("%5.2f", temp));
      });

    StackPane listPane = new StackPane();
    listPane.getChildren().add(list);

    primaryStage.setScene(new Scene(listPane, 800, 600));
    primaryStage.show();

  }

  private static String getWeatherData(URL url) throws IOException {
    try (BufferedReader r = new BufferedReader(new InputStreamReader(url.openStream()))) {
      return r.lines().collect(Collectors.joining("\n"));
    }
  }

}
