package at.jku.ssw.fp.sect10_5;

/**
 * Class representing weather data retrieved from the internet. 
 * 
 * @author Herbert Praehofer
 */
public class WeatherData {
  
  /** 
   * Creates an weather data object from a JSON string. 
   * @param data the JSON string
   * @return the weather data object 
   */
  public static WeatherData fromJSON(String data) {
    return new WeatherData(
        getDoubleValue(data, "temp"), 
        getDoubleValue(data, "humidity"), 
        getDoubleValue(data, "pressure"), 
        getDoubleValue(data, "speed"),
        getStringValue(data, "clouds"));
  }

  /** The temperature value */ 
  public final double temp; 

  /** The humidity value */ 
  public final double humidity; 

  /** The pressure value */ 
  public final double pressure; 

  /** The wind speed */ 
  public final double speed; 

  /** The cloud value */ 
  public final String clouds;
  
  /**
   * Constructor. 
   * @param temp the temperature value
   * @param humidity the humidity value
   * @param pressure the pressure value
   * @param speed the wind speed
   * @param clouds the clouding
   */
  private WeatherData(double temp, double humidity, 
      double pressure, double speed, String clouds) {
    super();
    this.temp = temp;
    this.humidity = humidity;
    this.pressure = pressure;
    this.speed = speed;
    this.clouds = clouds;
  }
	
	@Override
	public String toString() {
		return "WeatherData [temperature=" + temp + ", humidity=" + humidity + ", pressure=" + pressure
				+ ", windSpeed=" + speed + ", clouds=" + clouds + "]";
	}

	// --- private ---
	
	private static double getDoubleValue(String data, String key) {
		int start = findDataStart(data, key); 
		int end = Math.min(data.indexOf(",", start),data.indexOf("}", start)); 
		return Double.parseDouble(data.substring(start, end));
	}

	private static int findDataStart(String data, String key) {
		int i = data.indexOf(key) + key.length(); 
		while (!Character.isDigit(data.charAt(i))) {
			i++; 
		}
		return i;
	}

	private static String getStringValue(String data, String key) {
		int start = data.indexOf(key) + key.length() + 1; 
		while (data.charAt(start) != '"' ) {
			start++; 
		}
		int end = data.indexOf('"', start + 1); 
		return data.substring(start + 1, end);
	}
	
}