package at.jku.ssw.fp.parstream.performance;

import java.util.Arrays;
import java.util.OptionalInt;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp01_ArraySize {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	private final int[] intArr1_000 = CreateSupport.createIntArray(SIZES[0]);
	private final int[] intArr10_000 = CreateSupport.createIntArray(SIZES[1]);
	private final int[] intArr100_000 = CreateSupport.createIntArray(SIZES[2]);
	
	// 1_000 --- 

    @Benchmark @BenchmarkMode(Mode.Throughput) 
    public OptionalInt seqArrSize1_000() {
    	return Arrays.stream(intArr1_000).reduce((x, y) -> x + y);
    }

    @Benchmark @BenchmarkMode(Mode.Throughput) 
    public OptionalInt parArrSize1_000() {
    	return Arrays.stream(intArr1_000).parallel().reduce((x, y) -> x + y);
    }

	// 10_000 --- 

    @Benchmark @BenchmarkMode(Mode.Throughput) 
    public OptionalInt seqArrSize10_000() {
    	return Arrays.stream(intArr10_000).reduce((x, y) -> x + y);
    }

    @Benchmark @BenchmarkMode(Mode.Throughput) 
    public OptionalInt parArrSize10_000() {
    	return Arrays.stream(intArr10_000).parallel().reduce((x, y) -> x + y);
    }

	// 100_000 --- 

    @Benchmark @BenchmarkMode(Mode.Throughput) 
    public OptionalInt seqArrSize100_000() {
    	return Arrays.stream(intArr100_000).reduce((x, y) -> x + y);
    }

    @Benchmark @BenchmarkMode(Mode.Throughput)
    public OptionalInt parArrSize100_000() {
    	return Arrays.stream(intArr100_000).parallel().reduce((x, y) -> x + y);
    }

}
