package at.jku.ssw.fp.parstream.performance;

import java.util.Arrays;
import java.util.Collection;
import java.util.Optional;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp02_Lists {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	private final Integer[] array1_000 = CreateSupport.createIntegerArray(SIZES[0]);
	private final Collection<Integer> arrayList1_000 = CreateSupport.createArrayList(SIZES[0]);
	private final Collection<Integer> linkedList1_000 = CreateSupport.createLinkedList(SIZES[0]);

	private final Integer[] array10_000 = CreateSupport.createIntegerArray(SIZES[1]);
	private final Collection<Integer> arrayList10_000 = CreateSupport.createArrayList(SIZES[1]);
	private final Collection<Integer> linkedList10_000 = CreateSupport.createLinkedList(SIZES[1]);

	private final Integer[] array100_000 = CreateSupport.createIntegerArray(SIZES[2]);
	private final Collection<Integer> arrayList100_000 = CreateSupport.createArrayList(SIZES[2]);
	private final Collection<Integer> linkedList100_000 = CreateSupport.createLinkedList(SIZES[2]);

	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> array1_000() {
    	return Arrays.stream(array1_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> arrayList1000() {
    	return arrayList1_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> linkedList1000() {
    	return linkedList1_000.parallelStream().reduce((x, y) -> x + y);
    }
    
    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> array10_000() {
    	return Arrays.stream(array10_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> arrayList10_000() {
    	return arrayList10_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> linkedList10_000() {
    	return linkedList10_000.parallelStream().reduce((x, y) -> x + y);
    }

    // 100_000 ---
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> array100_000() {
    	return Arrays.stream(array100_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> arrayList100_000() {
    	return arrayList100_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> linkedList100_000() {
    	return linkedList100_000.parallelStream().reduce((x, y) -> x + y);
    }


}
