package at.jku.ssw.fp.parstream.performance;

import java.util.Collection;
import java.util.Optional;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp03_Collections {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	final Collection<Integer> arrayList1_000 = CreateSupport.createArrayList(SIZES[0]);
	final Collection<Integer> arrayList10_000 = CreateSupport.createArrayList(SIZES[1]);
	final Collection<Integer> arrayList100_000 = CreateSupport.createArrayList(SIZES[2]);
	
	final Collection<Integer> linkedList1_000 = CreateSupport.createLinkedList(SIZES[0]);
	final Collection<Integer> linkedList10_000 = CreateSupport.createLinkedList(SIZES[1]);
	final Collection<Integer> linkedList100_000 = CreateSupport.createLinkedList(SIZES[2]);
	
	final Collection<Integer> hashSet1_000 = CreateSupport.createHashSet(SIZES[0]);
	final Collection<Integer> hashSet10_000 = CreateSupport.createHashSet(SIZES[1]);
	final Collection<Integer> hashSet100_000 = CreateSupport.createHashSet(SIZES[2]);
	
	final Collection<Integer> treeSet1_000 = CreateSupport.createTreeSet(SIZES[0]);
	final Collection<Integer> treeSet10_000 = CreateSupport.createTreeSet(SIZES[1]);
	final Collection<Integer> treeSet100_000 = CreateSupport.createTreeSet(SIZES[2]);
	

	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> arrayList1_000() {
    	return arrayList1_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> linkedList1_000() {
    	return linkedList1_000.parallelStream().reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> hashSet1_000() {
    	return hashSet1_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> treeSet1_000() {
    	return treeSet1_000.parallelStream().reduce((x, y) -> x + y);
    }
    
    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> arrayList10_000() {
    	return arrayList10_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> linkedList10_000() {
    	return linkedList10_000.parallelStream().reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> hashSet10_000() {
    	return hashSet10_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> treeSet10_000() {
    	return treeSet10_000.parallelStream().reduce((x, y) -> x + y);
    }
    
    // 100_000 ---
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> arrayList100_000() {
    	return arrayList100_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> linkedList100_000() {
    	return linkedList100_000.parallelStream().reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> hashSet100_000() {
    	return hashSet100_000.parallelStream().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> treeSet100_000() {
    	return treeSet100_000.parallelStream().reduce((x, y) -> x + y);
    }

}
