package at.jku.ssw.fp.parstream.performance;

import java.util.Arrays;
import java.util.OptionalInt;
import java.util.stream.IntStream;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Thread)
public class Exp04_ArrayRangeIterate {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	int[] intArray1_000 = CreateSupport.createIntArray(SIZES[0]);
	final IntStream intRange1_000 = IntStream.range(1, SIZES[0]);
	final IntStream intIterate1_000 = IntStream.iterate(1, x -> x + 1).limit(SIZES[0]);
	
	final int[] intArray10_000 = CreateSupport.createIntArray(SIZES[1]);
	final IntStream intRange10_000 = IntStream.range(1, SIZES[1]);
	final IntStream intIterate10_000 = IntStream.iterate(1, x -> x + 1).limit(SIZES[1]);
	
	final int[] intArray100_000 = CreateSupport.createIntArray(SIZES[2]);
	final IntStream intRange100_000 = IntStream.range(1, SIZES[2]);
	final IntStream intIterate100_000 = IntStream.iterate(1, x -> x + 1).limit(SIZES[2]);
	
	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intArrayR1_000() {
    	return Arrays.stream(intArray1_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intRange1_000() {
    	return IntStream.range(1, SIZES[0]).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intIterate1_000() {
    	return IntStream.iterate(1, x -> x + 1).limit(SIZES[0]).parallel().reduce((x, y) -> x + y);
    }
    
    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intArray10_000() {
    	return Arrays.stream(intArray10_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intRange10_000() {
    	return IntStream.range(1, SIZES[1]).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intIterate10_000() {
    	return IntStream.iterate(1, x -> x + 1).limit(SIZES[1]).parallel().reduce((x, y) -> x + y);
    }
    
    // 100_000 ---
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intArray100_000() {
    	return Arrays.stream(intArray100_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intRange100_000() {
    	return IntStream.range(1, SIZES[2]).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intIterate100_000() {
    	return IntStream.iterate(1, x -> x + 1).limit(SIZES[2]).parallel().reduce((x, y) -> x + y);
    }

}
