package at.jku.ssw.fp.parstream.performance;

import java.util.Arrays;
import java.util.OptionalInt;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp05_Operations_Sequential {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	public static final int[] N_OPS = new int[] { 2, 10, 20, 50, 100 };
	
	private final int[] array1_000 = CreateSupport.createIntArray(1_000);
	private final int[] array10_000 = CreateSupport.createIntArray(10_000);
	private final int[] array100_000 = CreateSupport.createIntArray(100_000);
	
	private static final int sum(int x, int ops) {
		for (int i = 0; i < ops; i++) {
			if (x % 2 == 0) {
				x = x + i;
			} else {
				x = x - i;
			}
		}
		return x;
	}


	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array1_000_OPS_2() {
    	return Arrays.stream(array1_000).map(x -> sum(x, 2)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array1_000_OPS_10() {
    	return Arrays.stream(array1_000).map(x -> sum(x, 10)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array1_000_OPS_20() {
    	return Arrays.stream(array1_000).map(x -> sum(x, 20)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array1_000_OPS_50() {
    	return Arrays.stream(array1_000).map(x -> sum(x, 50)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array1_000_OPS_100() {
    	return Arrays.stream(array1_000).map(x -> sum(x, 100)).reduce((x, y) -> x + y);
    }
    
    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array10_000_OPS_2() {
    	return Arrays.stream(array10_000).map(x -> sum(x, 2)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array10_000_OPS_10() {
    	return Arrays.stream(array10_000).map(x -> sum(x, 10)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array10_000_OPS_20() {
    	return Arrays.stream(array10_000).map(x -> sum(x, 20)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array10_000_OPS_50() {
    	return Arrays.stream(array10_000).map(x -> sum(x, 50)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array10_000_OPS_100() {
    	return Arrays.stream(array10_000).map(x -> sum(x, 100)).reduce((x, y) -> x + y);
    }
    

    // 100_000 ---
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array100_000_OPS_2() {
    	return Arrays.stream(array100_000).map(x -> sum(x, 2)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array100_000_OPS_10() {
    	return Arrays.stream(array100_000).map(x -> sum(x, 10)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array100_000_OPS_20() {
    	return Arrays.stream(array100_000).map(x -> sum(x, 20)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array100_000_OPS_50() {
    	return Arrays.stream(array100_000).map(x -> sum(x, 50)).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt array100_000_OPS_100() {
    	return Arrays.stream(array100_000).map(x -> sum(x, 100)).reduce((x, y) -> x + y);
    }
    
}
