package at.jku.ssw.fp.parstream.performance;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp06_collects {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	private final Collection<Integer> arrayList1_000 = CreateSupport.createArrayList(SIZES[0]);
	private final Collection<Integer> arrayList10_000 = CreateSupport.createArrayList(SIZES[1]);
	private final Collection<Integer> arrayList100_000 = CreateSupport.createArrayList(SIZES[2]);

	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public List<Integer> arrayList1_000() {
    	return arrayList1_000.parallelStream().collect(Collectors.toCollection(() -> new ArrayList<Integer>()));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public List<Integer> linkedList1_000() {
    	return arrayList1_000.parallelStream().collect(Collectors.toCollection(() -> new LinkedList<Integer>()));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Set<Integer> hashSet1_000() {
    	return arrayList1_000.parallelStream().collect(Collectors.toCollection(() -> new HashSet<Integer>()));
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Set<Integer> treeSet1_000() {
    	return arrayList1_000.parallelStream().collect(Collectors.toCollection(() -> new TreeSet<Integer>()));
    }

    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public List<Integer> arrayList10_000() {
    	return arrayList10_000.parallelStream().collect(Collectors.toCollection(() -> new ArrayList<Integer>()));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public List<Integer> linkedList10_000() {
    	return arrayList10_000.parallelStream().collect(Collectors.toCollection(() -> new LinkedList<Integer>()));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Set<Integer> hashSet10_000() {
    	return arrayList10_000.parallelStream().collect(Collectors.toCollection(() -> new HashSet<Integer>()));
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Set<Integer> treeSet10_000() {
    	return arrayList10_000.parallelStream().collect(Collectors.toCollection(() -> new TreeSet<Integer>()));
    }

    // 100_000 ---
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public List<Integer> arrayList100_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toCollection(() -> new ArrayList<Integer>()));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public List<Integer> linkedList100_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toCollection(() -> new LinkedList<Integer>()));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Set<Integer> hashSet100_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toCollection(() -> new HashSet<Integer>()));
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Set<Integer> treeSet100_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toCollection(() -> new TreeSet<Integer>()));
    }

}
