package at.jku.ssw.fp.parstream.performance;

import java.util.ArrayList;
import java.util.Optional;
import java.util.Random;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Thread)
public class Exp07_find {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	private final ArrayList<Integer> arrayList1_000 = (ArrayList<Integer>)CreateSupport.createArrayList(SIZES[0]);
	private final ArrayList<Integer> arrayList10_000 = (ArrayList<Integer>)CreateSupport.createArrayList(SIZES[1]);
	private final ArrayList<Integer> arrayList100_000 = (ArrayList<Integer>)CreateSupport.createArrayList(SIZES[2]);
	private Random rand = new Random(); 
	
	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> findFirst1_000() {
    	int idx = rand.nextInt(1_000);
    	arrayList1_000.set(idx, -1); 
    	Optional<Integer> r = arrayList1_000.parallelStream().filter(x -> x < 0).findFirst();
    	arrayList1_000.set(idx, 1); 
    	return r; 
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> findAny1_000() {
    	int idx = rand.nextInt(1_000);
    	arrayList1_000.set(idx, -1); 
    	Optional<Integer> r = arrayList1_000.parallelStream().filter(x -> x < 0).findAny();
    	arrayList1_000.set(idx, 1); 
    	return r; 
    }

    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> findFirst10_000() {
    	int idx = rand.nextInt(10_000);
    	arrayList10_000.set(idx, -1); 
    	Optional<Integer> r = arrayList10_000.parallelStream().filter(x -> x < 0).findFirst();
    	arrayList10_000.set(idx, 1); 
    	return r; 
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> findAny10_000() {
    	int idx = rand.nextInt(10_000);
    	arrayList10_000.set(idx, -1); 
    	Optional<Integer> r = arrayList10_000.parallelStream().filter(x -> x < 0).findAny();
    	arrayList10_000.set(idx, 1); 
    	return r; 
    }

    // 100_000 ---
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> findFirst100_000() {
    	int idx = rand.nextInt(100_000);
    	arrayList100_000.set(idx, -1); 
    	Optional<Integer> r = arrayList100_000.parallelStream().filter(x -> x < 0).findFirst();
    	arrayList100_000.set(idx, 1); 
    	return r; 
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> findAny100_000() {
    	int idx = rand.nextInt(100_000);
    	arrayList100_000.set(idx, -1); 
    	Optional<Integer> r = arrayList100_000.parallelStream().filter(x -> x < 0).findAny();
    	arrayList100_000.set(idx, 1); 
    	return r; 
    }

}
