package at.jku.ssw.fp.parstream.performance;

import java.util.ArrayList;
import java.util.Optional;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp09_sorted {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	private final ArrayList<Integer> arrayList1_000 = (ArrayList<Integer>)CreateSupport.createArrayListPosNeg(SIZES[0]);
	private final ArrayList<Integer> arrayList10_000 = (ArrayList<Integer>)CreateSupport.createArrayListPosNeg(SIZES[1]);
	private final ArrayList<Integer> arrayList100_000 = (ArrayList<Integer>)CreateSupport.createArrayListPosNeg(SIZES[2]);

	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> noSorted1_000() {
    	return arrayList1_000.parallelStream().unordered().filter(x -> x < 0).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> sorted1_000() {
    	return arrayList1_000.parallelStream().unordered().filter(x -> x < 0).sorted().reduce((x, y) -> x + y);
    }
    
    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> noSorted10_000() {
    	return arrayList10_000.parallelStream().unordered().filter(x -> x < 0).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> sorted10_000() {
    	return arrayList10_000.parallelStream().unordered().filter(x -> x < 0).sorted().reduce((x, y) -> x + y);
    }
    

    // 100_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> noSorted100_000() {
    	return arrayList100_000.parallelStream().unordered().filter(x -> x < 0).reduce((x, y) -> x + y);
    }
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> sorted100_000() {
    	return arrayList100_000.parallelStream().unordered().filter(x -> x < 0).sorted().reduce((x, y) -> x + y);
    }
    
}
