package at.jku.ssw.fp.parstream.performance;

import java.util.Arrays;
import java.util.Optional;
import java.util.OptionalInt;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;


@State(Scope.Benchmark)
public class Exp10_boxing {
	
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };
	
	private final int[] intArr1_000 = CreateSupport.createIntArray(SIZES[0]);
	private final int[] intArr10_000 = CreateSupport.createIntArray(SIZES[1]);
	private final int[] intArr100_000 = CreateSupport.createIntArray(SIZES[2]);
	
	private final Integer[] integerArr1_000 = CreateSupport.createIntegerArray(SIZES[0]);
	private final Integer[] integerArr10_000 = CreateSupport.createIntegerArray(SIZES[1]);
	private final Integer[] integerArr100_000 = CreateSupport.createIntegerArray(SIZES[2]);
	
	// 1_000 --- 

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intArr1_000() {
    	return Arrays.stream(intArr1_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> integerArr1_000() {
    	return Arrays.stream(integerArr1_000).parallel().reduce((x, y) -> x + y);
    }

	// 10_000 --- 

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intArr10_000() {
    	return Arrays.stream(intArr10_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> integerArr10_000() {
    	return Arrays.stream(integerArr10_000).parallel().reduce((x, y) -> x + y);
    }

	// 100_000 --- 
    
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public OptionalInt intArr100_000() {
    	return Arrays.stream(intArr100_000).parallel().reduce((x, y) -> x + y);
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Optional<Integer> integerArr100_000() {
    	return Arrays.stream(integerArr100_000).parallel().reduce((x, y) -> x + y);
    }

}
