package at.jku.ssw.fp.parstream.performance;

import java.util.Collection;
import java.util.Map;
import java.util.stream.Collectors;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp11_toMap {

	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };

	private final Collection<Integer> arrayList1_000 = CreateSupport.createArrayList(SIZES[0]);
	private final Collection<Integer> arrayList10_000 = CreateSupport.createArrayList(SIZES[1]);
	private final Collection<Integer> arrayList100_000 = CreateSupport.createArrayList(SIZES[2]);

	// 1000 ---

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_1_000_S_10() {
    	return arrayList1_000.parallelStream().collect(Collectors.toMap(x -> x % 10, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_1_000_S_10() {
    	return arrayList1_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 10, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_1_000_S_100() {
    	return arrayList1_000.parallelStream().collect(Collectors.toMap(x -> x, x -> x % 100, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_1_000_S_100() {
    	return arrayList1_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 100, x -> 1, (s, ONE) -> s + 1));
    }

    // 10_000 ---

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_10_000_S_100() {
    	return arrayList10_000.parallelStream().collect(Collectors.toMap(x -> x % 100, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_10_000_S_100() {
    	return arrayList10_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 100, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_10_000_S_1000() {
    	return arrayList10_000.parallelStream().collect(Collectors.toMap(x -> x % 1000, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_10_000_S_1000() {
    	return arrayList10_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 1000, x -> 1, (s, ONE) -> s + 1));
    }

    // 100_000 ---

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_100_000_S_100() {
    	return arrayList100_000.parallelStream().collect(Collectors.toMap(x -> x % 100, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_100_000_S_100() {
    	return arrayList100_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 100, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_100_000_S_1000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toMap(x -> x % 1000, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_100_000_S_1000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 1000, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toMap_100_000_S_10_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toMap(x -> x % 10_000, x -> 1, (s, ONE) -> s + 1));
    }

    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, Integer> toConcurrentMap_100_000_S_10_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.toConcurrentMap(x -> x % 10_000, x -> 1, (s, ONE) -> s + 1));
    }


}
