package at.jku.ssw.fp.parstream.performance;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import at.jku.ssw.fp.parstream.performance.util.CreateSupport;

@State(Scope.Benchmark)
public class Exp12_groupingBy {
	
	private static final int[] N_GROUPS = new int[] { 100, 1_000, 10_000 };
	public static final int[] SIZES = new int[] { 1_000, 10_000, 100_000 };

	private final Collection<Integer> arrayList1_000 = CreateSupport.createArrayList(SIZES[0]);
	private final Collection<Integer> arrayList10_000 = CreateSupport.createArrayList(SIZES[1]);
	private final Collection<Integer> arrayList100_000 = CreateSupport.createArrayList(SIZES[2]);

	// 1000 ---
	
    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingBy_1_000_Group_100() {
    	return arrayList1_000.parallelStream().collect(Collectors.groupingBy(x -> x % N_GROUPS[0]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingByConcurrent_1_000_Group_100() {
    	return arrayList1_000.parallelStream().collect(Collectors.groupingByConcurrent(x -> x % N_GROUPS[0]));
    }

    // 10_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingBy_10_000_Group_100() {
    	return arrayList10_000.parallelStream().collect(Collectors.groupingBy(x -> x % N_GROUPS[0]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingByConcurrent_10_000_Group_100() {
    	return arrayList10_000.parallelStream().collect(Collectors.groupingByConcurrent(x -> x % N_GROUPS[0]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingBy_10_000_Group_1000() {
    	return arrayList10_000.parallelStream().collect(Collectors.groupingBy(x -> x % N_GROUPS[1]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingByConcurrent_10_000_Group_1000() {
    	return arrayList10_000.parallelStream().collect(Collectors.groupingByConcurrent(x -> x % N_GROUPS[1]));
    }

    // 100_000 ---

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingBy_100_000_Group_1_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.groupingBy(x -> x % N_GROUPS[1]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingByConcurrent_100_000_Group_1_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.groupingByConcurrent(x -> x % N_GROUPS[1]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingBy_100_000_Group_10_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.groupingBy(x -> x % N_GROUPS[2]));
    }

    @Benchmark 
    @BenchmarkMode(Mode.Throughput) 
    @Warmup(iterations = 2)
    @Measurement(iterations = 2)
    @Fork(2)
    public Map<Integer, List<Integer>> groupingByConcurrent_100_000_Group_10_000() {
    	return arrayList100_000.parallelStream().collect(Collectors.groupingByConcurrent(x -> x % N_GROUPS[2]));
    }

}
