package at.jku.ssw.fp.parstream.performance.util;

import static at.jku.ssw.fp.parstream.performance.util.Constants.COUNTED_MEAS;
import static at.jku.ssw.fp.parstream.performance.util.Constants.N_MEAS;

import java.util.Arrays; 

public class Measurement {
	
	private int n = 0; 
	private long[] measures = new long[N_MEAS]; 
	private double mean = -1; 
	private double stdDev = -1; 
	private long min = -1; 
	private boolean cached = false; 
	
	public final String name; 
	
	private long startTime; 

	public Measurement(String name) {
		super();
		this.name = name; 
		for (int i = 0; i < measures.length; i++) {
			measures[i] = Long.MAX_VALUE; 
		}
	}

	public void addMeasurement(long measure) {
		if (cached) {
			cached = false; 
		}
		measures[n] = measure; 
		n++; 
	}
	
	public int getNumber() {
		return n; 
	}
	
	public double getMean() {
		if (! cached) {
			computeResults(); 
		}
		return mean; 
	}
	
	private void computeResults() {
		Arrays.sort(measures); 
		// min
		min = measures[0]; 
		// mean
		double sum = 0; 
		int c = (n < COUNTED_MEAS ? n : COUNTED_MEAS); 
		for (int i = 0; i < c; i++) {
			sum += measures[i]; 
		}
		mean = sum / c; 
		// stdDev
		double sumSqrs = 0.0; 
		for (int i = 0; i < c; i++) {
			double diff = measures[i] - mean; 
			sumSqrs += diff * diff; 
		}
		stdDev = Math.sqrt(sumSqrs / (c - 1));
		cached = true; 				
	}
	
	public long getMin() {
		if (! cached) {
			computeResults(); 
		}
		return min;
	}
	
	public double getStdDev() {
		if (! cached) {
			computeResults(); 
		}		
		return stdDev; 
	}
	
	public void start() {
		startTime = System.nanoTime(); 
	}

	public void measure() {
		addMeasurement(System.nanoTime() - startTime);		
	}

	public void print() {
		System.out.format("Measurement: %s Runs %,d, counted %,d %n   min = %,12d ns %n   mean = %,12.0f ns %n   std dev = %,12.0f %n", 
				name, n, (n < COUNTED_MEAS ? n : COUNTED_MEAS), getMin(), getMean(), getStdDev()); 
//		for (int i = 0; i < 10; i++) {
//			System.out.format("   %,12d ", measures[i]);
//		}
		System.out.format("%n");
	}

}
