package at.jku.ssw.fp.parstream.performance.util;

import java.io.PrintStream;
import java.util.SortedMap;
import java.util.TreeMap;

public class PerfStatistics {

	private final String name;
	private final SortedMap<Integer, Measurement> meass;

	public PerfStatistics(String name, int[] sizes) {
		this.name = name;
		this.meass = new TreeMap<Integer, Measurement>();
		for (int size : sizes) {
			meass.put(size, new Measurement(String.format("Measures %s of size %d", name, size)));
		}
	}

	public void startMeasurement(int size) {
		meass.get(size).start();
	}

	public void measure(int size) {
		meass.get(size).measure();
	}

	public void print() {
		System.out.println();
		System.out.println(name);
		for (Measurement m : meass.values()) {
			m.print();
		}
	}

	public void toCSV(PrintStream out) {
		out.println(name);
		out.println(String.format("Size;Mean;Std dev;Min"));
		for (int size : meass.keySet()) {
			Measurement m = meass.get(size);
			out.println(String.format("%d;%f;%f;%d", size, m.getMean(), m.getStdDev(), m.getMin()));
		}
		out.println();
	}

}
