package at.jku.ssw.fp.airline;

import java.time.LocalDate;

import at.jku.ssw.fp.util.Util;

/**
 * Class for retailers of flights in the flight example.
 *
 * @author Herbert Praehofer
 *
 */
public class Agency implements Retailer {

  /**Array with some agency objects */
  public static final Agency[] AGENCIES = { new Agency("Ruefa"), new Agency("TUI") };

  /** The name of the retailer */
  public final String name;

  /**
   * Constructor setting the name.
   *
   * @param name the name of the agency.
   */
  public Agency(String name) {
    super();
    this.name = name;
  }

  /**
   * Returns an offer for a flight.
   *
   * @param flight the flight
   * @return the offer for the flight
   */
  public Offer getOffer(Flight flight) {
    return new Offer(this, flight, 120 + RAND.nextInt(120));
  }

  /**
   * Finds a flight provided by an airline.
   *
   * @param airline the airline
   * @param from the origin of the flight
   * @param to the destination  of the flight
   * @param date the date of the flight
   * @return the flight object provided by the airline
   */
  public Flight findFlight(Airline airline,
      String from, String to, LocalDate date) {
    Util.log(String.format("Agency %s findFlight", name));

    return airline.findFlight(from, to, date);
  }

  /**
   * Returns a string representation of the agency.
   *
   * @return the string representation
   */
  @Override
  public String toString() {
    return "Agency [name=" + name + "]";
  }

}
