package at.jku.ssw.fp.airline;

import java.time.LocalDateTime;

/**
 * Class representing a flight.
 *
 * @author Herbert Praehofer
 *
 */
public class Flight {

  /** The origin of the flight */
  public final String from;

  /** The destination of the flight */
  public final String to;

  /** The airline */
  public final Airline airline;

  /** The flight number */
  public final String flightNumber;

  /** The date and time of the departure */
  public final LocalDateTime departs;

  /** The price of the flight */
  public final int price;

  /**
   * Constructor creating a flight.
   *
   * @param from the origin of the flight
   * @param to the destination of the flight
   * @param airline the airline of the flight
   * @param flightNumber the number of the flight
   * @param departs the departure date and time of the flight
   * @param price the price of the flight
   */
  public Flight(String from, String to, Airline airline, String flightNumber, LocalDateTime departs, int price) {
    super();
    this.from = from;
    this.to = to;
    this.airline = airline;
    this.flightNumber = flightNumber;
    this.departs = departs;
    this.price = price;
  }



  /**
   * Returns the hash code.
   *
   * @return the hash code
   */
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((airline == null) ? 0 : airline.hashCode());
    result = prime * result + ((flightNumber == null) ? 0 : flightNumber.hashCode());
    result = prime * result + ((from == null) ? 0 : from.hashCode());
    result = prime * result + ((to == null) ? 0 : to.hashCode());
    return result;
  }


  /**
   * Indicates if this object is equal to the provided object.
   *
   * @param obj the other object
   * @return {@code true} if this object is equal to the provided object
   */
  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    Flight other = (Flight) obj;
    if (airline == null) {
      if (other.airline != null)
        return false;
    } else if (!airline.equals(other.airline))
      return false;
    if (flightNumber == null) {
      if (other.flightNumber != null)
        return false;
    } else if (!flightNumber.equals(other.flightNumber))
      return false;
    if (from == null) {
      if (other.from != null)
        return false;
    } else if (!from.equals(other.from))
      return false;
    if (to == null) {
      if (other.to != null)
        return false;
    } else if (!to.equals(other.to))
      return false;
    return true;
  }



  /**
   * Returns a string representation of the flight
   */
  @Override
  public String toString() {
    return "FlightConn [from=" + from + ", to=" + to + ", airline=" + airline + ", flightNumber=" + flightNumber
        + ", departs=" + departs + ", price=" + price + "]";
  }

}
