package at.jku.ssw.fp.airline;

/**
 * An offer for a flight
 *
 * @author Herbert Praehofer
 *
 */
public class Offer {

  /** The flight of the offer */
  public final Flight flight;

  /** The price of the offer */
  public final int price;

  /** The retailer offering the flight */
public final Retailer retailer;

  /**
   * Constructor creating an offer.
   * @param retailer the retailer of the offer
   * @param flight the flight
   * @param price the price
   */
  public Offer(Retailer retailer, Flight flight, int price) {
    super();
    this.retailer = retailer;
    this.flight = flight;
    this.price = price;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((flight == null) ? 0 : flight.hashCode());
    result = prime * result + price;
    result = prime * result + ((retailer == null) ? 0 : retailer.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    Offer other = (Offer) obj;
    if (flight == null) {
      if (other.flight != null)
        return false;
    } else if (!flight.equals(other.flight))
      return false;
    if (price != other.price)
      return false;
    if (retailer == null) {
      if (other.retailer != null)
        return false;
    } else if (!retailer.equals(other.retailer))
      return false;
    return true;
  }


}
