package at.jku.ssw.fp.airline;

import java.util.Random;
import java.util.concurrent.CompletableFuture;

/**
 * Interface for retailers in the airline example.
 * Retailers can provide offers synchronously and asynchronously.
 *
 * @author Herbert Praehofer
 *
 */
public interface Retailer {

  /**
   * Random number generator.
   */
  public static final Random RAND = new Random(7);

  /**
   * Returns an offer for a flight in a synchronous way.
   *
   * @param flight the flight
   * @return the offer for the flight
   */
  public Offer getOffer(Flight flight);

  /**
   * Returns a completable future for an offer for a flight.
   *
   * @param flight the flight
   * @return the completable future for the offer for the flight
   */
  default CompletableFuture<Offer> getOfferAsync(Flight flight) {
    return CompletableFuture.supplyAsync(() -> getOffer(flight));
  }

}
