package at.jku.ssw.fp.sect02_1;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import at.jku.ssw.fp.person.Person;
import at.jku.ssw.fp.person.Professor;
import at.jku.ssw.fp.person.Student;

/**
 * Demos for generics.
 *
 * @author Herbert Praehofer
 *
 */
public class Demo1_ApplyFunction {

  @SuppressWarnings("unused")
  public static void main(String[] args) {

    Function<Object, String> toStringFn = new Function<Object, String>() {
      public String apply(Object o) {
        return o.toString();
      }
    };

    // Object someInfo = applyFunction(toStringFn, new Person("Mike"));
    Object someInfo = applyFunction(toStringFn, new Person("Mike"));

    var persons = new ArrayList<Person>();
    List<Person> persons2 = Collections.emptyList();
    var persons3 = Collections.<Person>emptyList();

    List<Student> students = new ArrayList<Student>();
    List<Professor> professors = new ArrayList<Professor>();
    List<Person> persons4 = either(students, professors);

  }

  /**
   * Applies a function object on an object value.
   *
   * @param <A> the type of the value
   * @param <B> the type of the result
   * @param fn the function to apply
   * @param obj the object value
   * @return the result of the application
   */
  public static <A, B> B applyFunction(Function<? super A, ? extends B> fn, A obj) {
    return fn.apply(obj);
  }

  /**
   * Returns the first list if not empty, otherwise the second list
   * @param <A> the common base type of the elements of th two lists
   * @param first the first list
   * @param second the second list
   * @return the first list if not empty, otherwise the second list
   */
  @SuppressWarnings("unchecked")
  static <A> List<A> either(List<? extends A> first, List<? extends A> second) {
    if (!first.isEmpty())
      return (List<A>) first;
    else
      return (List<A>) second;
  }

}
