package at.jku.ssw.fp.sect02_2;

/**
 * Interface extending {@link Comparable} with relational operator methods.
 *
 * @author Herbert Praehofer
 *
 * @param <T> the type of the values to compare
 *
 */
public interface Ordered<T> extends Comparable<T> {
  // inherits  compareTo(T o);

  /**
   * Tests if this object is less than the given argument value.
   *
   * @param o the object to compare with
   *
   * @return {@code true} if this is less than the other object
   */
  public default boolean less(T o) {
    return this.compareTo(o) < 0;
  }

  /**
   * Tests if this object is greater than the given argument value.
   *
   * @param o the object to compare with
   * @return {@code true} if this is greater than the other object
   */
  public default boolean greater(T o) {
    return this.compareTo(o) > 0;
  }

  /**
   * Tests if this object is less than or equal to the given argument value.
   *
   * @param o the object to compare with
   * @return {@code true} if this is less than or equal to the other object
   */
  public default boolean lessEqual(T o) {
    return this.compareTo(o) <= 0;
  }

  /**
   * Tests if this object is greater than or equal to the given argument value.
   *
   * @param o the object to compare with
   * @return {@code true} if this is greater than or equal to the other object
   */
  public default boolean greaterEqual(T o) {
    return this.compareTo(o) >= 0;
  }

}
