package at.jku.ssw.fp.sect02_3;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;


/**
 * Demos showing lambda expressions with exceptions.
 *
 * @author Herbert Praehofer
 *
 */
public class Demo2_LambdaWithExceptions {

  public static void main(String[] args) throws IOException {
    // applyFunction(path -> Files.readAllLines(path), Paths.get(""));

    List<String> lines = applyFunctionWithExcpt(path -> Files.readAllLines(path), Paths.get("files/text.txt"));
    System.out.println(lines);

    Optional<List<String>> optionalLines = applyFunction(path -> {
      try {
        return Optional.of(Files.readAllLines(path));
      } catch (IOException e) {
        return Optional.empty();
      }
    }, Paths.get("files/text.txt"));
    System.out.println(optionalLines);

  }

  static <T, R> R applyFunction(Function<? super T, ? extends R> fn, T obj) {
    return fn.apply(obj);
  }

  static <T, R, X extends Exception> R applyFunctionWithExcpt(FunctionWithExcpt<T, R, X> fn, T obj) throws X {
    return fn.apply(obj);
  }

}
