package at.jku.ssw.fp.sect03_1;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

/**
 * Demos of memoization.
 *
 * @author Herbert Praehofer
 */
public class Demo3_Memoize {

  public static void main(String[] args) {
    System.out.println(isPrime(17));
    System.out.println(isPrimeWithCache(17));
    System.out.println(isPrimeWithCache(17));

    Function<Long, Boolean> isPrimeWithMemo = memoize(n -> isPrime(n));
    System.out.println(isPrimeWithMemo.apply(17L));
    System.out.println(isPrimeWithMemo.apply(17L));

  }

  static Map<Long, Boolean> primeCache = new HashMap<Long, Boolean>();

  static boolean isPrime(long n) {
    for (int i = 2; i * i <= n; i++) {
      if (n % i == 0)
        return false;
    }
    return true;
  }

  public static boolean isPrimeWithCache(long n) {
    Boolean isPrimeN = primeCache.get(n);
    if (isPrimeN != null)
      return isPrimeN;
    isPrimeN = isPrime(n);
    primeCache.put(n, isPrimeN);
    return isPrimeN;
  }

  static <T, R> Function<T, R> memoize(Function<T, R> fn) {
    final Map<T, R> cache = new HashMap<T, R>();
    return t -> {
      R r = cache.get(t);
      if (r == null) {
        r = fn.apply(t);
        cache.put(t, r);
      }
      return r;
    };
  }

  public static <T, R> Function<T, R> memoize2(Function<T, R> fn) {
    final Map<T, R> cache = new HashMap<T, R>();
    return t -> cache.computeIfAbsent(t, fn);
  }

}
