package at.jku.ssw.fp.sect04_0;

import java.util.List;
import java.util.function.Consumer;

/**
 * Demo showing the implementation of a higher-order function forEach
 *
 * @author Herbert Praehofer
 */
public class Demo_ForEach {

  /**
   * Prints the elements of a list.
   *
   * @param <E> the type of elements
   * @param list the list of elements to print.
   */
  static <E> void printEach(List<E> list) {
    for (E elem : list) {
      System.out.println(elem.toString());
    }
  }

  /**
   * Executes an action for the elements of a list.
   *
   * @param <E> the type of elements
   * @param list the list of elements
   * @param action the function object to execute on the elements
   */
  static <E> void forEach(List<E> list, Consumer<E> action) {
    for (E elem : list) {
      action.accept(elem);
    }
  }

  public static void main(String[] args) {

    var list = List.of("A", "B");

    printEach(list);

    forEach(list, elem -> System.out.println(elem));

  }

}
