package at.jku.ssw.fp.sect04_2.fp;

import java.util.ArrayList;
import java.util.List;

import at.jku.ssw.fp.util.Pair;
import at.jku.ssw.fp.util.Tuple2;

/**
 * Demo showing computation on result records.
 *
 * @author Herbert Praehofer
 */
public class Demo_Results {

  private static double avrgScore(Results r) {
    return r.compute(Tuple2.of(0, 0.0), (numSum, score) -> Tuple2.of(numSum.v1 + 1, numSum.v2 + score),
        numSum -> numSum.v2 / numSum.v1);
  }

  public static void main(String[] args) {
    ResultsList resultsList = new ResultsList();
    resultsList.addStudent("1", "Hans");
    resultsList.addStudent("2", "Fritz");
    resultsList.addStudent("3", "Anna");

    // 1 2 3 4 5 6 7 8 9 10
    resultsList.setScores("1", 7, 8, 2, 7, 9, 9, 7, 6, 7, 2);
    resultsList.setScores("2", 8, 6, 8, 1, 0, 0, 8, 9, 8, 3);
    resultsList.setScores("3", 9, 9, 9, 6, 5, 9, 8, 6, 9, 9);

    resultsList.setGradingFunction(record -> {
      if (record.countScores(s -> s >= 4) < 8.0)
        return Grade.INSUFFICIENT;
      if (avrgScore(record) < 5.0)
        return Grade.INSUFFICIENT;
      if (avrgScore(record) < 6.5)
        return Grade.SUFFICIENT;
      if (avrgScore(record) < 8.0)
        return Grade.SATISFACTORY;
      if (avrgScore(record) < 9.0)
        return Grade.GOOD;
      else
        return Grade.EXCELLENT;
    });

    // Test
    int solvedHans = resultsList.countScores("1", score -> score >= 4);
    int nMaxScoreHans = resultsList.countScores("1", score -> score == 10);
    Grade gradeHans = resultsList.getGrade("1");
    double avrgHans = resultsList.compute("1", Pair.of(0, 0.0),
        (numSum, score) -> score >= 4 ? Pair.of(numSum.fst + 1, numSum.snd + score) : numSum,
        numSum -> numSum.snd / numSum.fst);
    System.out.println("\nHans");
    System.out.println(solvedHans);
    System.out.println(nMaxScoreHans);
    System.out.println(gradeHans);
    System.out.println(avrgHans);

    int solvedFritz = resultsList.countScores("2", score -> score >= 4);
    int nMaxScoreFritz = resultsList.countScores("2", score -> score == 10);
    Grade gradeFritz = resultsList.getGrade("2");
    double avrgFritz = resultsList.compute("2", Pair.of(0, 0.0),
        (numSum, score) -> score >= 4 ? Pair.of(numSum.fst + 1, numSum.snd + score) : numSum,
        numSum -> numSum.snd / numSum.fst);

    System.out.println("\nFritz");
    System.out.println(solvedFritz);
    System.out.println(nMaxScoreFritz);
    System.out.println(gradeFritz);
    System.out.println(avrgFritz);

    int solvedAnna = resultsList.countScores("3", score -> score >= 4);
    int nMaxScoreAnna = resultsList.countScores("3", score -> score == 10);
    Grade gradeAnna = resultsList.getGrade("3");
    double avrgAnna = resultsList.compute("3", Pair.of(0, 0.0),
        (numSum, score) -> score >= 4 ? Pair.of(numSum.fst + 1, numSum.snd + score) : numSum,
        numSum -> numSum.snd / numSum.fst);
    System.out.println("\nAnna");
    System.out.println(solvedAnna);
    System.out.println(nMaxScoreAnna);
    System.out.println(gradeAnna);
    System.out.println(avrgAnna);

    int nSolved3 = resultsList.count(record -> record.getScore(3) >= 4);
    System.out.println("\nAssignment 3: ");
    System.out.println(nSolved3);

    List<String> sufficientSolved = resultsList.compute(new ArrayList<String>(), (list, record) -> {
      if (record.countScores(s -> s >= 4) >= 8)
        list.add(record.getName());
      return list;
    }, list -> list);
    System.out.println(sufficientSolved);

  }

}
