package at.jku.ssw.fp.sect04_2.imp;

/**
 * Class representing the results of a student. 
 * 
 * @author Herbert Praehofer 
 */
public class Results {

  /** Number of assignments */
  private static final int N = 10;

  /** Maximum points for an assignment */
  private static final int MAX_SCORE = 10;

  /** The student id */
  private final String id;
  
  /** The name of the student */
  private final String name;
  
  /** The array of scores for the different assignments */
  private final int[] scores;

  /**
   * Constructor initalizing id and name.
   * @param id the student id
   * @param name the student name
   */
  public Results(String id, String name) {
    this.id = id;
    this.name = name;
    this.scores = new int[N];
  }

  /**
   * Returns the id of the student. 
   * @return the id of the student
   */
  public String getStdId() {
    return id;
  }

  /**
   * Returns the name of the student. 
   * @return the name of the student
   */
  public String getName() {
    return name;
  }

  /**
   * Returns the score of an assignment
   * @param assignment the number of the assignment. 
   * @return the score of the assignment
   */
  public int getScore(int assignment) {
    assert assignment >= 1 && assignment <= N;
    return scores[assignment - 1];
  }

  /**
   * Returns the array of the scores for the {@link N} assignments. 
   * @return the array of the scores
   */
  public int[] getScores() {
    return scores;
  }

  /**
   * Sets the score of an assignment. 
   * @param assignment the number of the assignment 
   * @param score the score for the assignment
   */
  public void setScore(int assignment, int score) {
    assert assignment >= 1 && assignment <= N;
    assert score >= 0 && assignment <= N;
    scores[assignment - 1] = score;
  }

  /** 
   * Sets the scores for the {@link N} assignments. 
   * @param scores the array with the scores
   */
  public void setScores(int... scores) {
    assert (scores.length == 10);
    for (int i = 1; i <= N; i++) {
      setScore(i, scores[i - 1]);
    }
  }

  /**
   * Counts the assignments which are solved. 
   * An assignment is solved if 40% of the points have been achieved. 
   * @return the number of solved assignments
   */
  public int countSolved() {
    int n = 0;
    for (int pts : scores) {
      if (pts >= MAX_SCORE * 0.4) {
        n++;
      }
    }
    return n;
  }

  /**
   * Computes the average of the scores. 
   * @return the average of the scores
   */
  public double avrgScores() {
    double sum = 0;
    for (int pts : scores) {
      sum += pts;
    }
    return sum / N;
  }

  /**
   * Computes the grade from the scores.
   * 
   * @return the grade
   */
  public Grade getGrade() {
    if (countSolved() < 8.0)
      return Grade.INSUFFICIENT;
    if (avrgScores() < 5.0)
      return Grade.INSUFFICIENT;
    if (avrgScores() < 6.5)
      return Grade.SUFFICIENT;
    if (avrgScores() < 8.0)
      return Grade.SATISFACTORY;
    if (avrgScores() < 9.0)
      return Grade.GOOD;
    else
      return Grade.EXCELLENT;
  }

}
