package at.jku.ssw.fp.sect04_3;

import static at.jku.ssw.fp.sect04_3.SearchAlgosGeneric.bestFirstSearch;
import static at.jku.ssw.fp.sect04_3.SearchAlgosGeneric.breadthFirstSearch;
import static at.jku.ssw.fp.sect04_3.SearchAlgosGeneric.depthFirstSearch;

import java.util.Optional;

/**
 * Demo showing search in a graph with the generic search algorithms.
 *
 * @author Herbert Praehofer
 */
public class Demo2_SearchAlgosGeneric {

  public static int nodesVisited = 0;


  public static void main(String[] args) {

    // 5 4 3
    // A------D----E---H
    // 7| \ 2/ |
    // | 2\ / |1
    // C---B---G---F
    // 2 4 2

    Graph<String> graph = new Graph<>();

    Node<String> a = graph.addNode("A");
    Node<String> b = graph.addNode("B");
    Node<String> c = graph.addNode("C");
    Node<String> d = graph.addNode("D");
    Node<String> e = graph.addNode("E");
    Node<String> f = graph.addNode("F");
    Node<String> g = graph.addNode("G");
    Node<String> h = graph.addNode("H");

    graph.biConnect(a, b, 2);
    graph.biConnect(a, c, 7);
    graph.biConnect(a, d, 5);
    graph.biConnect(b, c, 2);
    graph.biConnect(b, d, 2);
    graph.biConnect(b, g, 4);
    graph.biConnect(d, e, 4);
    graph.biConnect(e, f, 1);
    graph.biConnect(e, h, 3);
    graph.biConnect(g, f, 2);

    // -- depth first

    Optional<Node<String>> optAtoH_DepthF = depthFirstSearch(node -> node.getSuccNodes(), n -> n.getValue().equals("H"),
        a);
    System.out.println("Depth H = " + optAtoH_DepthF);
    System.out.println("---------------------------------\n");

    Optional<Node<String>> optAtoX_DepthF = depthFirstSearch(node -> node.getSuccNodes(), n -> n.getValue().equals("X"),
        a);
    System.out.println("Depth X = " + optAtoX_DepthF);
    System.out.println("---------------------------------\n");

    // breadth first

    Optional<Node<String>> optAtoH_BreadthF = breadthFirstSearch(node -> node.getSuccNodes(),
        n -> n.getValue().equals("H"), a);
    System.out.println("Breadth H = " + optAtoH_BreadthF);
    System.out.println("---------------------------------\n");

    Optional<Node<String>> optAtoX_BreadthF = breadthFirstSearch(node -> node.getSuccNodes(),
        n -> n.getValue().equals("X"), a);
    System.out.println("Breadth X = " + optAtoX_BreadthF);
    System.out.println("---------------------------------\n");

    Optional<Node<String>> optAtoH_BestF = bestFirstSearch(node -> node.getSuccNodes(), n -> 1,
        n -> n.getValue().equals("H"), a);
    System.out.println("Best H = " + optAtoH_BestF);
    System.out.println("---------------------------------\n");

    Optional<Node<String>> optAtoX_BestF = bestFirstSearch(node -> node.getSuccNodes(), n -> 1,
        n -> n.getValue().equals("X"), a);
    System.out.println("Best X = " + optAtoX_BestF);
    System.out.println("---------------------------------\n");

  }

}
