package at.jku.ssw.fp.sect04_3;

/**
 * Class representing an edge in a graph.
 * An edge has a weight.
 *
 * @author Herbert Praehofer
 *
 * @param <V> the type of the value of the nodes
 */
public class Edge<V> {

  /** The source node of the edge */
  private final Node<V> from;

  /** The target node of the edge */
  private final Node<V> to;

  /** The weight of the edge */
  private int weight;

  /**
   * Constructor setting source and target and weight.
   *
   * @param from the source node
   * @param to the target node
   * @param weight the weight
   */
  Edge(Node<V> from, Node<V> to, int weight) {
    super();
    this.from = from;
    this.to = to;
    this.weight = weight;
  }

  /**
   * Returns the source node
   * @return the source node
   */
  public Node<V> getFrom() {
    return from;
  }

  /**
   * Returns the target node
   * @return the target node
   */
  public Node<V> getTo() {
    return to;
  }

  /**
   * Returns the weight
   * @return the weight
   */
  public int getWeight() {
    return weight;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((from == null) ? 0 : from.hashCode());
    result = prime * result + ((to == null) ? 0 : to.hashCode());
    return result;
  }

  @SuppressWarnings("unchecked")
  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    Edge<V> other = (Edge<V>) obj;
    if (from == null) {
      if (other.from != null)
        return false;
    } else if (!from.equals(other.from))
      return false;
    if (to == null) {
      if (other.to != null)
        return false;
    } else if (!to.equals(other.to))
      return false;
    return true;
  }

  @Override
  public String toString() {
    return "Edge [from=" + from + ", to=" + to + ", weight=" + weight + "]";
  }

}