package at.jku.ssw.fp.sect04_3;

import java.util.ArrayList;
import java.util.List;

/** Graph with nodes and edges */
public class Graph<V> {

  /** The list of nodes */
  private final List<Node<V>> nodes;
  
  /** The list of edges */
  private final List<Edge<V>> edges;

  /**
   * Constructor initializing the lists of nodes and edges with empty lists. 
   */
  public Graph() {
    nodes = new ArrayList<>();
    edges = new ArrayList<>();
  }

  /**
   * Adds a node with a given value to the graph. 
   * 
   * @param value the value in the node
   * @return the node added
   */
  public Node<V> addNode(V value) {
    Node<V> n = new Node<V>(value);
    nodes.add(n);
    return n;
  }

  /** 
   * Connects two nodes with a given weight. 
   * 
   * @param from the source node
   * @param to the destination node
   * @param weight the weight
   * @return the edge
   */
  public Edge<V> connect(Node<V> from, Node<V> to, int weight) {
    Edge<V> e = Node.connect(from, to, weight);
    edges.add(e);
    return e;
  }

  /** 
   * Makes a bidirectional connection of two nodes with a given weight. 
   * 
   * @param from the first node
   * @param to the second node
   * @param weight the weight   */
  public void biConnect(Node<V> from, Node<V> to, int weight) {
    Edge<V> e = Node.connect(from, to, weight);
    edges.add(e);
    Edge<V> eBack = Node.connect(to, from, weight);
    edges.add(eBack);
  }

  /**
   * Returns the list of nodes of this graph.
   * @return the list of nodes
   */
  public List<Node<V>> getNodes() {
    return nodes;
  }

  /**
   * Returns the list of edges of this graph.
   * @return the list of edges
   */
  public List<Edge<V>> getEdges() {
    return edges;
  }

}
