package at.jku.ssw.fp.sect04_3.puzzle;

import static at.jku.ssw.fp.sect04_3.SearchAlgosGeneric.bestFirstSearch;
import static at.jku.ssw.fp.sect04_3.SearchAlgosGeneric.breadthFirstSearch;
import static at.jku.ssw.fp.sect04_3.SearchAlgosGeneric.nodesVisited;

import java.util.Optional;

/**
 * Demo for solving the puzzle by the generic versions of the search algorithms. 
 * 
 * @author Herbert Praehofer
 */
public class Demo2_PuzzleSearchGeneric {

  public static void main(String[] args) {

    Puzzle start = Puzzle.from(8, 0, 3, 2, 7, 5, 4, 6, 1); // Puzzle.shuffled();
    start.print(System.out);

    // breadth first

    Optional<Puzzle> solOptBreadth = breadthFirstSearch(p -> p.getNextStates(), p -> p.isSolution(), start);
    System.out.println("Nodes visited = " + nodesVisited);
    if (solOptBreadth.isPresent()) {
      Puzzle sol = solOptBreadth.get();
      System.out.println("Breadth-first: solution found with " + sol.getMoveHistory().size() + " moves");
      for (Puzzle p : sol.getStateHistory()) {
        p.print(System.out);
      }
      sol.print(System.out);
    } else {
      System.out.println("Breadth-first: no solution");
    }
    
    // best first

    Optional<Puzzle> solOptBestFirst = bestFirstSearch(p -> p.getNextStates(), p -> p.assess(), p -> p.isSolution(),
        start);

    System.out.println("Nodes visited = " + nodesVisited);
    if (solOptBestFirst.isPresent()) {
      Puzzle sol = solOptBestFirst.get();
      System.out.println("Best-first: solution found with " + sol.getMoveHistory().size() + " moves");
      for (Puzzle p : sol.getStateHistory()) {
        p.print(System.out);
      }
      sol.print(System.out);
    } else {
      System.out.println("Best-first: no solution");
    }

  }

}
