package at.jku.ssw.fp.sect04_3.puzzle;

/**
 * A move in a puzzle game. 
 * 
 * @author Herbert Praehofer
 *
 */
public final class Move {

  /** Size of the puzzle */ 
  public static final int N = Puzzle.N;

  /** 
   * Creates a move from a position to a position 
   * @param from the from position 
   * @param to the to position 
   * @return the move 
   */
  public static Move of(Pos from, Pos to) {
    return new Move(from, to);
  }

  /** the from position */
  public final Pos from;
  
  /** the to position  */
  public final Pos to;

  /**
   * Private constructor. 
   * 
   * @param from the from position 
   * @param to the to position 
   */
  private Move(Pos from, Pos to) {
    super();
    this.from = from;
    this.to = to;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((from == null) ? 0 : from.hashCode());
    result = prime * result + ((to == null) ? 0 : to.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    Move other = (Move) obj;
    if (from == null) {
      if (other.from != null)
        return false;
    } else if (!from.equals(other.from))
      return false;
    if (to == null) {
      if (other.to != null)
        return false;
    } else if (!to.equals(other.to))
      return false;
    return true;
  }

  @Override
  public String toString() {
    return "[" + from + " -> " + to + "]";
  }

}
