package at.jku.ssw.fp.sect04_3.puzzle;

/**
 * A position in a puzzle. 
 * 
 * @author Herbert Praehofer
 *
 */
public final class Pos {

  /** Size of the puzzle */
  public static final int N = Puzzle.N;

  /**
   * Creats a position. 
   * 
   * @param row the row index
   * @param col the column index
   * @return the position 
   */
  public static Pos of(int row, int col) {
    return new Pos(row, col);
  }

  /** the row index */
  public final int row;
  
  /** the column index */
  public final int col;

  /**
   * Private constructor. 
   * @param row the row index
   * @param col the column index
   */
  private Pos(int row, int col) {
    super();
    this.row = row;
    this.col = col;
  }

  /**
   * Tests if this position is within the range of the puzzle. 
   * @return {@code true} if this position is within the range of the puzzle
   */
  public boolean isValid() {
    return row >= 0 && row < N && col >= 0 && col < N;
  }

  /**
   * Tests if the given position is a neighbor of this position 
   * @param other the other position 
   * @return {@code true} if the given position is a neighbor 
   */
  public boolean isNeighbor(Pos other) {
    if (!other.isValid())
      return false;
    int dx = Math.abs(this.col - other.col);
    int dy = Math.abs(this.row - other.row);
    return (dx == 1 && dy == 0) || (dx == 0 && dy == 1);
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + col;
    result = prime * result + row;
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    Pos other = (Pos) obj;
    if (col != other.col)
      return false;
    if (row != other.row)
      return false;
    return true;
  }

  @Override
  public String toString() {
    return "(" + row + "/" + col + ")";
  }

}
