package at.jku.ssw.fp.sect04_4.cmd_fp;

import java.util.ArrayDeque;
import java.util.Deque;

/**
 * Handler for commands. Allows undo of commands.
 *
 * @author Herbert Prähofer
 *
 */
public class CmdHandler {

  /** Singleton command handler */
  private static CmdHandler instance = new CmdHandler();

  /**
   * Gets the singleton command handler.
   *
   * @return the singleton command handler
   */
  public static CmdHandler getInstance() {
    return instance;
  }

  /** Undo stack */
  private final Deque<Command> undoStack;

  /**
   * Private constructor
   */
  private CmdHandler() {
    super();
    this.undoStack = new ArrayDeque<>();
  }

  /**
   * Executes the command and stores it in the undo stack.
   *
   * @param cmd the command to execute
   */
  public void doCmd(Command cmd) {
    cmd.doAction.run();
    undoStack.addFirst(cmd);
  }

  /**
   * Undoes the last command from the undo stack.
   */
  public void undoCmd() {
    if (undoStack.isEmpty())
      return;
    undoStack.pop().undoAction.run();
  }

}
