package at.jku.ssw.fp.sect04_4.cmd_nofp;

/**
 * Command to add a text.
 *
 * @author Herbert Prähofer
 *
 */
public class AddTextCommand implements Command {

  /** The string builder where the text is added. */
  private final StringBuilder text;

  /** The position where the text is added. */
  private final int pos;

  /** The text to add. */
  private final String textToAdd;

  /**
   * Constructor setting string builder, position and text.
   *
   * @param text The string builder where the text should be added
   * @param pos The position where the text should be added
   * @param textToAdd The text to add
   */
  public AddTextCommand(StringBuilder text, int pos, String textToAdd) {
    this.text = text;
    this.pos = pos;
    this.textToAdd = textToAdd;
  }

  /**
   * Executes the command.
   */
  @Override
  public void doCmd() {
    text.insert(pos, textToAdd);
  }

  /**
   * Undoes the command.
   */
  @Override
  public void undoCmd() {
    text.delete(pos, pos + textToAdd.length());
  }

}
