package at.jku.ssw.fp.sect04_4.cmd_nofp;

import java.util.ArrayDeque;
import java.util.Deque;

/**
 * Handler for commands. Allows undo and redo of commands.
 *
 * @author Herbert Prähofer
 *
 */
public class CmdHandler {


  /** Singleton command handler */
  private static CmdHandler instance = new CmdHandler();

  /**
   * Gets the singleton command handler.
   *
   * @return the singleton command handler
   */
  public static CmdHandler getInstance() {
    return instance;
  }


  /** Undo stack */
  private final Deque<Command> undoStack = new ArrayDeque<>();

  /** Redo stack */
  private final Deque<Command> redoStack = new ArrayDeque<>();

  /**
   * Executes the command and stores it in the undo stack.
   *
   * @param cmd the command to execute
   */
  public void doCmd(Command cmd) {
    cmd.doCmd();
    undoStack.addFirst(cmd);
  }

  /**
   * Undoes the last command from the undo stack.
   */
  public void undoCommand() {
    if (undoStack.isEmpty())
      return;
    undoStack.pop().undoCmd();
  }

  /**
   * Undoes the last command from the redo stack.
   */
  public void redoCmd() {
    if (redoStack.isEmpty()) {
      return;
    }
    Command cmd = redoStack.removeFirst();
    cmd.doCmd();
    undoStack.addFirst(cmd);
  }

}
