package at.jku.ssw.fp.sect04_4.cmd_nofp;


/**
 * Command to delete a text.
 *
 * @author Herbert Prähofer
 *
 */
public class DeleteTextCommand implements Command {

  /** The string builder where the text is deleted. */
  private final StringBuilder text;

  /** The position where the text is deleted. */
  private final int pos;

  /** The length of the text to delete. */
  private final int length;

  /** The deleted text. */
  private final String textDeleted;

  /**
   * Constructor setting string builder, position and length of the text to delete.
   *
   * @param text The string builder where the text should be deleted
   * @param pos The position where the text should be deleted
   * @param length  The length of the text to delete
   */
  public DeleteTextCommand(StringBuilder text, int pos, int length) {
    this.text = text;
    this.pos = pos;
    this.length = length;
    this.textDeleted = text.substring(pos, length);
  }

  /**
   * Executes the command.
   */
  @Override
  public void doCmd() {
    text.delete(pos, pos + length);
  }

  /**
   * Undoes the command.
   */
  @Override
  public void undoCmd() {
    text.insert(pos, textDeleted);
  }

}
