package at.jku.ssw.fp.sect04_4.visitor_fp;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * Variable bindings. Uses a map from variable name to Boolean value.
 *
 * @author Herbert Prähofer
 *
 */
public class Bindings {

  /** Map from variable name to Boolean value. */
  private final Map<String, Boolean> map;

  /**
   * Constructor creating an empty map.
   */
  public Bindings() {
    this.map = new HashMap<>();
  }

  /**
   * Returns the Boolean value for the variable.
   * Returns {@code false} if variable does not exist.
   *
   * @param name the variable name
   * @return the value; {@code false} if variable does not exist
   */
  public boolean get(String name) {
    return Optional.ofNullable(map.get(name)).orElse(false);
  }

  /**
   * Sets the value for the variable
   * @param name the variable name
   * @param value the value
   */
  public void set(String name, boolean value) {
    map.put(name, value);
  }

}
