package at.jku.ssw.fp.sect04_4.visitor_fp;

import at.jku.ssw.fp.sect04_4.visitor_fp.Expr.And;
import at.jku.ssw.fp.sect04_4.visitor_fp.Expr.Lit;
import at.jku.ssw.fp.sect04_4.visitor_fp.Expr.Not;
import at.jku.ssw.fp.sect04_4.visitor_fp.Expr.Or;
import at.jku.ssw.fp.sect04_4.visitor_fp.Expr.Var;

/**
 * Demo showing the application of apply for an expression.
 *
 * @author Herbert Praehofer
 */
public class Demo_ExprApply {

  public static void main(String[] args) {

    // (true && x) || ! (y && false)
    Expr expr = new Or(new And(new Lit(true), new Var("x")), new Not(new Not(new And(new Var("y"), new Lit(false)))));

    System.out.println("expr = " + mkString(expr));

    Bindings bds = new Bindings();
    bds.set("x", false);
    bds.set("y", true);
    boolean result = eval(expr, bds);
    System.out.println("result = " + result);

    Expr sExpr = simplify(expr);
    System.out.println("simplified = " + mkString(sExpr));
    boolean sResult = eval(expr, bds);
    System.out.println("sResult = " + sResult);

    print(expr, "");

  }

  static boolean eval(Expr expr, Bindings bds) {
    return expr.apply(l -> l.value, v -> bds.get(v.name), and -> eval(and.left, bds) && eval(and.right, bds),
        or -> eval(or.left, bds) || eval(or.right, bds), not -> !eval(not.sub, bds));
  }

  public static String mkString(Expr expr) {
    return expr.apply(
        (Lit lit) -> String.valueOf(lit.value),
        (Var vbl) -> vbl.name,
        (And and) -> "(" + mkString(and.left) + " && " +
               mkString(and.right) + ")",
        (Or or)   -> "(" +  mkString(or.left) + " || " +
               mkString(or.right) + ")",
        (Not not) -> "(! " + mkString(not.sub) + ")"
      );
  }


  static Expr simplify(Expr expr) {
    return expr.apply(l -> l, v -> v, and -> {
      Expr sLeft = simplify(and.left);
      Expr sRight = simplify(and.right);
      if (isLitValue(sLeft, false) || isLitValue(sRight, false)) {
        return new Lit(false);
      } else if (isLitValue(sLeft, true)) {
        return sRight;
      } else if (isLitValue(sRight, true)) {
        return sLeft;
      } else {
        return new And(sLeft, sRight);
      }
    }, or -> {
      Expr sLeft = simplify(or.left);
      Expr sRight = simplify(or.right);
      if (isLitValue(sLeft, true) || isLitValue(sRight, true)) {
        return new Lit(true);
      } else if (isLitValue(sLeft, false)) {
        return sRight;
      } else if (isLitValue(sRight, false)) {
        return sLeft;
      } else {
        return new And(sLeft, sRight);
      }
    }, not -> {
      Expr sSub = simplify(not.sub);
      if (isLitValue(sSub, true)) {
        return new Lit(false);
      } else if (isLitValue(sSub, false)) {
        return new Lit(true);
      } else {
        return new Not(sSub);
      }
    });
  }

  static boolean isLitValue(Expr expr, boolean value) {
    return expr.apply(l -> l.value == value, v -> false, and -> false, or -> false, not -> false);
  }

  static void print(Expr expr, String ident) {
    expr.accept(l -> System.out.println(ident + "i am a lit"), v -> System.out.println(ident + "i am a var"), a -> {
      System.out.println(ident + "i am an and");
      print(a.left, ident + "  ");
      print(a.right, ident + "  ");
    }, o -> {
      System.out.println(ident + "i am an or");
      print(o.left, ident + "  ");
      print(o.right, ident + "  ");
    }, n -> {
      System.out.println(ident + "i am a not");
      print(n.sub, ident + "  ");
    });

  }

}
