package at.jku.ssw.fp.sect04_4.visitor_nofp;

import at.jku.ssw.fp.sect04_4.visitor_nofp.Expr.And;
import at.jku.ssw.fp.sect04_4.visitor_nofp.Expr.Lit;
import at.jku.ssw.fp.sect04_4.visitor_nofp.Expr.Not;
import at.jku.ssw.fp.sect04_4.visitor_nofp.Expr.Or;
import at.jku.ssw.fp.sect04_4.visitor_nofp.Expr.Var;

/**
 * Interface for visitors of expressions.
 *
 * @author Herbert Prähofer
 *
 * @param <R> the return value of the visit methods.
 */
public interface Visitor<R> {

  /**
   * Visits a literal.
   *
   * @param lit the literal to visit
   * @return the return value
   */
  public R visitLit(Lit lit);

  /**
   * Visits a variable.
   *
   * @param vbl the variable to visit
   * @return the return value
   */
  public R visitVar(Var vbl);

  /**
   * Visits a not-expression.
   *
   * @param not the not-expression to visit
   * @return the return value
   */
  public R visitNot(Not not);

  /**
   * Visits an and-expression.
   *
   * @param and the and-expression to visit
   * @return the return value
   */
  public R visitAnd(And and);

  /**
   * Visits an or-expression.
   *
   * @param or the or-expression to visit
   * @return the return value
   */
  public R visitOr(Or or);

}
