package at.jku.ssw.fp.sect04_5;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.otherwise;
import static at.jku.ssw.fp.util.When.when;

import java.util.Optional;
import java.util.Scanner;

import at.jku.ssw.fp.util.CondFunction;

/**
 * Demo showing use of conditional function. 
 * 
 * @author Herbert Praehofer
 */
public class Demo5_CondFunction {

  @SuppressWarnings("unused")
  public static void main(String[] args) {
    

    try (Scanner scanner = new Scanner(System.in)) {
      while (true) {

        // input of number

        System.out.print("Input your number: ");
        int number = scanner.nextInt();

        CondFunction<Integer, Integer> signOfNegative = 
            CondFunction.of(n -> n < 0, n -> -1);

        Optional<Integer> optSign = signOfNegative.apply(number);
        System.out.println("The optional sign is the number is " + optSign);

        // positive

        int pos = (number < 0) ? -number : number;

        CondFunction<Integer, Integer> posFn = CondFunction.<Integer, Integer>of(n -> n < 0, n -> -n).orElse(n -> n);

        Optional<Integer> optPos = posFn.apply(number);
        System.out.println("The positive value of is " + optPos);

        // signum

        int sign = number < 0 ? -1 : (number == 0) ? 0 : +1;

        CondFunction<Integer, Integer> signum = 
            CondFunction.<Integer, Integer>of(n -> n < 0, n -> -1)
                    .orElseIf(n -> n == 0, n -> 0)
                    .orElse(n -> +1);


        optSign = signum.apply(number);

        System.out.println("The optional sign is the number is " + optSign);

        // description

        CondFunction<Integer, String> descrCondFn = CondFunction.of((Integer n) -> n < 0, (Integer n) -> "negative")
            .orElseIf((Integer n) -> n == 0, (Integer n) -> "zero")
            .orElseIf((Integer n) -> n > 0 && n < 10, (Integer n) -> "single-digit")
            .orElse((Integer n) -> "multi-digit");

        String descr = descrCondFn.apply(number).orElse("UNKNOWN");
        System.out.println("The number is " + descr);

        // using select

        pos = when(number, is(n -> n < 0, n -> -n), otherwise(n -> n));

        int sign1 = 
            when( number, 
              is(n -> n < 0, n -> -1), 
              is(n -> n == 0, n -> 0), 
              otherwise(n -> +1) 
            );
        System.out.println("The sign is " + sign1);
        
        descr = 
            when(number, 
                is(n -> n < 0, n -> "negative"), 
                is(n -> n == 0, n -> "zero"),
                is(n -> n > 0 && n < 10, n -> "single-digit"), 
                otherwise(n -> "multi-digit")
             );
        System.out.println("The number is " + descr);

      }
    } catch (Exception e) {
    }
  }

}
