package at.jku.ssw.fp.sect04_5;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.when;

import at.jku.ssw.fp.expr.Expr;
import at.jku.ssw.fp.expr.Expr.And;
import at.jku.ssw.fp.expr.Expr.Lit;
import at.jku.ssw.fp.expr.Expr.Not;
import at.jku.ssw.fp.expr.Expr.Or;
import at.jku.ssw.fp.expr.Expr.Var;
import at.jku.ssw.fp.util.TypeTestFunction;

/**
 * Demo showing use of type test function. 
 * 
 * @author Herbert Praehofer
 */
public class Demo6_TypeTestFunction {

  public static void main(String[] args) {
    
    TypeTestFunction.of(
        Lit.class, (Lit lit) -> lit.value ? "true" : "false");


    // (true && x) || ! (y && false)
    Expr expr = new Or(new And(new Lit(true), new Var("x")), new Not(new Not(new And(new Var("y"), new Lit(false)))));

    System.out.println(mkString(expr));
    System.out.println(mkString2(expr));
    System.out.println(mkString3(expr));

  }

  static String mkString(Expr expr) {
    return 
        when(expr, 
          is(e -> e instanceof Lit, 
             e -> String.valueOf(((Lit)e).value)), 
          is(e -> e instanceof Var, 
             e -> ((Var)e).name), 
          is(e -> e instanceof And, 
             e -> {
               And and = (And)e; 
               return "(" + mkString(and.left) + " && " +
                      mkString(and.right) + ")";
             }), 
        is(e -> e instanceof Or, e -> {
          Or or = (Or) e;
          return "(" + mkString(or.left) + " || " + mkString(or.right) + ")";
        }), 
        is(e -> e instanceof Not, e -> {
          Not not = (Not) e;
          return "(! " + mkString(not.sub) + ")";
        })
      );
  }

  public static String mkString2(Expr expr) {
    return when(expr, is(Lit.class, (Lit lit) -> lit.value ? "true" : "false"), is(Var.class, (Var var) -> var.name),
        is(And.class, (And and) -> "(" + mkString(and.left) + " && " + mkString(and.right) + ")"),
        is(Or.class, (Or or) -> "(" + mkString(or.left) + " || " + mkString(or.right) + ")"),
        is(Not.class, (Not not) -> "(! " + mkString(not.sub) + ")"));
  }

  public static String mkString3(Expr expr) {
    return when(expr, 
      is(Lit.class, lit -> lit.value ? "true" : "false"), 
      is(Var.class, vbl -> vbl.name), 
      is(And.class, and -> "("+ mkString(and.left) + " && "+ 
                           mkString(and.right)+ ")"), 
      is(Or.class,  or  -> "(" +  mkString(or.left) + " || " + 
                           mkString(or.right) + ")"), 
      is(Not.class, not -> "(! " + mkString(not.sub) + ")")
    );
  }


}
