package at.jku.ssw.fp.sect04_6;

import static at.jku.ssw.fp.sect04_6.Lazy.find;
import static at.jku.ssw.fp.sect04_6.Lazy.generate;
import static at.jku.ssw.fp.sect04_6.Lazy.iterate;
import static at.jku.ssw.fp.sect04_6.Lazy.iterator;
import static at.jku.ssw.fp.sect04_6.Lazy.words;

import java.io.IOException;
import java.io.Reader;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.Optional;
import java.util.Random;;

/**
 * Demo showing lazy iterator. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_LazyIterator {

  private static final Random RAND = new Random();
  private static final int N = 10;

  public static void main(String[] args) throws IOException {

    System.out.println("\ngenerate:");

    Iterator<Integer> randIterator = generate(() -> RAND.nextInt(100));
    double sum = 0;
    for (int i = 0; i < N; i++) {
      System.out.println(randIterator.hasNext());
      int z = randIterator.next();
      System.out.println(z);
      sum += z;
    }
    System.out.println("Avrg = " + sum / N);

    Optional<Integer> randEven = find(randIterator, i -> i % 2 == 0);
    System.out.println("Even rand = " + randEven);

    System.out.println("\niterate:");

    Iterator<Long> pot2Iterator = iterator(1L, x -> x * 2);

    for (int i = 0; i < N; i++) {
      System.out.println(pot2Iterator.hasNext());
      long z = pot2Iterator.next();
      System.out.println(z);
    }

    Optional<Long> bigPot2 = find(iterate(1L, x -> x * 2), p -> p > 100_000_000_000_000L);
    System.out.println(bigPot2);

    System.out.println("\nwordsInFile:");

    Reader reader = Files.newBufferedReader(Paths.get("files/text.txt"));
    Iterable<String> words = words(reader);

    for (String word : words) {
      System.out.println(word);
    }
    reader.close();

    System.out.println("\nfind:");

    reader = Files.newBufferedReader(Paths.get("files/text.txt"));
    Optional<String> wordWithA = find(words(reader), word -> word.startsWith("a"));
    System.out.println(wordWithA);

    reader.close();

  }

}
