package at.jku.ssw.fp.sect04_6;

import static at.jku.ssw.fp.sect04_6.Lazy.find;
import static at.jku.ssw.fp.sect04_6.Lazy.iterate;
import static at.jku.ssw.fp.sect04_6.Lazy.iterator;

import java.io.IOException;
import java.util.Iterator;
import java.util.Optional;

/**
 * Demo showing computation of square root using lazy sequences. 
 * 
 * @author Herbert Praehofer
 */
public class Demo2_LazySqrt {

  private static double EPSILON = 1.0E-12;

  public static Iterator<Double> sqrtIterator(double x) {
    return iterator(x / 2.0, ai -> (ai + x / ai) / 2);
  }

  public static Iterable<Double> sqrtIterable(double x) {
    return iterate(x / 2.0, a -> (a + x / a) / 2);
  }

  public static Iterable<Double> sqrtApprox(double x) {
    return iterate(x / 2.0, ai -> (ai + x / ai) / 2);
  }

  public static Optional<Double> findSqrt(double x) {
    return find(sqrtApprox(x), r -> Math.abs(r * r - x) < EPSILON);
  }

  public static double sqrt(double x) {
    return find(sqrtIterable(x), r -> Math.abs(r * r - x) < EPSILON).get();
  }

  public static void main(String[] args) throws IOException {
    for (double x = 1.0; x <= 100.0; x = x + 1.0) {
      double x2 = sqrtImperative(x);
      System.out.format("%nSqrt of %.1f = %.12f, squared = %.12f", x, x2, x2 * x2);
    }
  }

  public static double sqrtImperative(double x) {
    double a = x / 2.0;
    while (Math.abs(a * a - x) >= EPSILON) {
      a = (a + x / a) / 2;
    }
    return a;
  }

}
