package at.jku.ssw.fp.sect05_2;

import java.awt.Component;
import java.util.Arrays;
import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.function.Function;
import java.util.function.Predicate;

/**
 * Demo showing function composition in {@link Function}, {@link Predicate} and {@link Comparator}. 
 * 
 * @author Herbert Praehofer
 */
public class Demo_FunctionComposition {

  @SuppressWarnings("unused")
  public static void main(String[] args) {

    Function<String, String[]> wordsFn = s -> s.split(" ");
    Function<String[], String[]> sortFn = words -> {
      Arrays.sort(words.clone());
      return words;
    };

    Function<String, String[]> sortWordsFn = wordsFn.andThen(sortFn);
    String[] sortedWords = 
        sortWordsFn.apply("this is a sentence with words");

    for (String s : sortedWords)
      System.out.println(s);

    Predicate<Component> isVisible = Component::isVisible;  
    Predicate<Component> hasFocus = Component::hasFocus; 
    Predicate<Component> isValid = Component::isValid; 
    Predicate<Component> p = 
        (isVisible.and(hasFocus.negate())).or(isValid.negate());

    SortedSet<Person> personSet = new TreeSet<>(
        (p1, p2) -> p1.getName().compareTo(p2. getName()));


    Comparator<String> stringComparator = Comparator.naturalOrder();
    Comparator<Person> personComparator = Comparator.naturalOrder();
    
    Comparator<Person> nameComparator = 
        Comparator.comparing(person -> person.getName());
    Comparator<Person> ageComparator = 
        Comparator.comparingInt(Person::getAge);


    Comparator<Person> personComparator1 = 
        nameComparator.thenComparing(ageComparator);

    Comparator<Person> personComparator2 = 
        nameComparator.thenComparing(Person::getAge);

    Comparator<Person> personComparator3 = 
        Comparator.comparing(Person::getName)
                  .thenComparing(Person::getFirstName)
                  .thenComparing(Person::getAge)
                  .reversed();


  }

}
