package at.jku.ssw.fp.sect05_2.workflow;

import static at.jku.ssw.fp.sect05_2.workflow.Step.repeat;
import static at.jku.ssw.fp.sect05_2.workflow.Treatment.BAKE;
import static at.jku.ssw.fp.sect05_2.workflow.Treatment.COOL;
import static at.jku.ssw.fp.sect05_2.workflow.Treatment.EAT;
import static at.jku.ssw.fp.sect05_2.workflow.Treatment.HEAT;
import static at.jku.ssw.fp.sect05_2.workflow.Treatment.STIR;

/**
 * Demo showing building workflow from functional steps. 
 * 
 * @author Herbert Praehofer
 */
public class Demo_Recipe {

  public static void main(String[] args) {

    Step<Object, Ingr> getMilk = x -> Ingr.of("Milk", 0.3); 
    Step<Object, Ingr> getFlour = x -> Ingr.of("Flour", 0.2); 
    Step<Object, Ingr> getButter = x -> Ingr.of("Butter",0.15);
    Step<Object, Ingr> getEggs = x -> Ingr.of("Egg", 5); 
    Step<Object, Ingr> getShoko = x -> Ingr.of("Shoko", 0.1);

    var prepareDough = 
        getEggs
          .combine(getMilk, (e, m) -> e.mix(m))
          .combine(getFlour, (em, f) -> em.mix(f))
          .combine(getButter, (emf, b) -> emf.mix(b))
          .then(repeat(4, emfb -> emfb.process(STIR)));

    var heatShoko = 
        getShoko
          .then(shoko -> shoko.process(HEAT)); 

    var makeCake = 
        prepareDough
          .combine(heatShoko, (dough, hs) -> dough.mix(hs))
          .then(dough -> dough.process(STIR))
          .then(dough -> dough.process(BAKE))
          .then(cake -> cake.process(COOL)); 

    var makeThenEat = 
        makeCake
          .then(repeat(16, cake -> cake.process(EAT))); 

    var result = makeThenEat.exec(); 

    System.out.println(result);

  }

}
