package at.jku.ssw.fp.sect05_2.workflow;

/**
 * Basic ingredients for a product produced by a workflow.
 *
 * @author Herbert Praehofer
 *
 */
public class Ingr extends Stuff {

  /**
   * Creates an ingredient with name and amount.
   * @param name the name
   * @param amount the amount
   * @return the ingredient
   */
  public static Ingr of(String name, double amount) {
    return new Ingr(name, amount);
  }

  /** The name of the ingredient. */
  private final String name;

  /** The amount of the ingredient. */
  private final double amount;

  /**
   * Private constructor initializing name and amount of the ingredient.
   *
   * @param name the name of the ingredient
   * @param amount the amount of the ingredient
   */
  private Ingr(String name, double amount) {
    super();
    this.name = name;
    this.amount = amount;
  }

  /**
   * Gets the name of this ingredient.
   *
   * @return the name
   */
  public String getName() {
    return name;
  }

  /**
   * Gets the amount of this ingredient.
   *
   * @return the amount
   */
  public double getAmount() {
    return amount;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    long temp;
    temp = Double.doubleToLongBits(amount);
    result = prime * result + (int) (temp ^ (temp >>> 32));
    result = prime * result + ((getName() == null) ? 0 : getName().hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    Ingr other = (Ingr) obj;
    if (Double.doubleToLongBits(amount) != Double.doubleToLongBits(other.amount))
      return false;
    if (getName() == null) {
      if (other.getName() != null)
        return false;
    } else if (!getName().equals(other.getName()))
      return false;
    return true;
  }

  @Override
  public String toString() {
    return "Ingrediant [name=" + getName() + ", amount=" + amount + "]";
  }

}
