package at.jku.ssw.fp.sect05_2.workflow;

import java.util.ArrayList;
import java.util.List;

/**
 * A product which consists of various stuff and has some treatments.
 *
 * @author Herbert Praehofer
 *
 */
public class Product extends Stuff {

  /**
   * Creates a product from various stuff.
   *
   * @param ingredients the ingredients for the product
   * @return the product
   */
  public static Product of(Stuff... ingredients) {
    return new Product(ingredients);
  }

  /**
   * List of ingredients.
   */
  protected final List<Stuff> ingredients;

  /**
   * Constructor initializing the ingredients of the product.
   *
   * @param ingredients the ingredients of the product
   */
  protected Product(Stuff[] ingredients) {
    super();
    this.ingredients = List.of(ingredients);
  }

  /**
   * Constructor initializing the ingredients and the treatments of the product.
   *
   * @param ingredients the ingredients of the product
   * @param treatments the treatments of the product
   */
  protected Product(Stuff[] ingredients, Treatment[] treatments) {
    super(treatments);
    this.ingredients = List.of(ingredients);
  }

  /**
   * Gets the ingredients of this product.
   *
   * @return the ingredients
   */
  public List<Stuff> getIngrediants() {
    return ingredients;
  }

  /**
   * Creates a new product by mixing a stuff to this product.
   *
   * @param stuff the added stuff
   * @return the new product
   */
  @Override
  public Product mix(Stuff stuff) {
    var ingrds = new ArrayList<>();
    ingrds.addAll(ingredients);
    ingrds.add(stuff);
    return new Product(ingrds.toArray(new Stuff[0]));
  }

  @Override
  public String toString() {
    return "Mixture [ingrediants=" + ingredients + "]";
  }

}
