package at.jku.ssw.fp.sect05_2.workflow;

import java.util.List;

/**
 * Base class for ingredients and products.
 * Stuff contains a list of treatments.
 *
 * @author Herbert Praehofer
 *
 */
public abstract class Stuff {

  /**
   * The list of treatments of this stuff
   */
  protected final List<Treatment> treatments;

  /**
   * Constructor initializing the list of treatments to be empty.
   */
  protected Stuff() {
    this.treatments = List.<Treatment>of();
  }

  /**
   * Constructor initializing the treatments.
   *
   * @param treatments the treatments
   */
  protected Stuff(Treatment[] treatments) {
    this.treatments = List.of(treatments);
  }

  /**
   * Creates a product by combining this stuff and a given stuff.
   * @param stuff the stuff to combine with
   * @return the produce resulting from the combination
   */
  public Product mix(Stuff stuff) {
    return Product.of(this, stuff);
  }

  /**
   * Processes this stuff with a treatment.
   *
   * @param treatment the treatment to treat this stuff
   * @return the treated product
   */
  public Product process(Treatment treatment) {
    return new Product(new Stuff[] { this }, new Treatment[] { treatment });
  }

}
