package at.jku.ssw.fp.sect05_3.parser;

import static at.jku.ssw.fp.sect05_3.parser.Parser.alt;
import static at.jku.ssw.fp.sect05_3.parser.Parser.anyToken;
import static at.jku.ssw.fp.sect05_3.parser.Parser.mult;
import static at.jku.ssw.fp.sect05_3.parser.Parser.opt;
import static at.jku.ssw.fp.sect05_3.parser.Parser.token;

import at.jku.ssw.fp.coll.FList;
import at.jku.ssw.fp.util.Pair;

/**
 * Demo showing building combinator parser.  
 * 
 * @author Herbert Praehofer
 */
public class Demo1_Parser {

  static Parser<String> notTkn = token("!");
  static Parser<String> orTkn = token("||");
  static Parser<String> andTkn = token("&&");

  @SuppressWarnings("unused")
  public static void main(String[] args) {

    FList<String> tokens = FList.of("true", "false");
    Result<? extends String> result = anyToken.apply(tokens); 
    System.out.println("anyToken(true false) --> " + result);

    Parser<String> trueToken = 
        anyToken.filter(t -> t.equals("true"));
    Parser<String> trueToken2 = token("true");
    
    Result<String> result1 = trueToken.apply(tokens);
    System.out.println("anyToken(true false).filter(t -> t.equals(true) --> " + result1);
    Result<String> result2 = trueToken.apply(result1.rest);
    System.out.println("anyToken(true false).filter(t -> t.equals(true) --> " + result2);

    Parser<Boolean> trueParser = token("true").map(t -> true); 
    Parser<Boolean> falseParser = token("false").map(t -> false);

    Parser<Pair<Boolean, Boolean>> trueFalseParser = 
        trueParser.seq(falseParser); 


    Result<Pair<Boolean, Boolean>> result4 =  
        trueFalseParser.apply(tokens); 


    Parser<Boolean> booleanParser = alt(trueParser, falseParser);

    Parser<FList<Boolean>> multTrueParser = mult(trueParser);
    tokens = FList.of("true true true false".split(" "));
    Result<? extends FList<Boolean>> multTrues = multTrueParser.apply(tokens);
    System.out.println(multTrues);
    tokens = FList.of("false".split(" "));
    multTrues = multTrueParser.apply(tokens);
    System.out.println(multTrues);

    var boolSeqParser = 
        opt(falseParser)
          .seq(mult(trueParser))
          .seq(falseParser)
          .seq (alt(trueParser, falseParser));


    var boolSeqParser2 = opt(falseParser).seq(mult(trueParser)).seq(falseParser).seq(alt(trueParser, falseParser));
    tokens = FList.of("false true true true false true".split(" "));
    System.out.println(boolSeqParser2.apply(tokens));

    tokens = FList.of("true true true false true".split(" "));
    System.out.println(boolSeqParser.apply(tokens));
    tokens = FList.of("true false true".split(" "));
    System.out.println(boolSeqParser.apply(tokens));

    var optMultFalseParser = opt(falseParser).seq(mult(trueParser)).seq(falseParser);
    tokens = FList.of("false false true".split(" "));
    System.out.println(optMultFalseParser.apply(tokens));

    FList<String> tokens2 = 
        FList.of("|| && true x ! && y false".split(" "));  

    var fmultTrueParser = mult(trueParser);
    var fmultResult = fmultTrueParser.apply(tokens2);
    System.out.println(fmultResult);

  }

}
