package at.jku.ssw.fp.sect05_4.dsl;

/**
 * Implementation of a proportional controller.
 *
 * @author Herbert Praehofer
 *
 */
public class PropCtrl implements Action {

  /** Proportional constant */
  private final double Kp;

  /** Variable for the setpoint */
  private final DVar setpoint;

  /** Variable for the measured value */
  private final DVar measured;

  /** Variable for the controller output */
  private final DVar output;

  /**
   * Constructor.
   *
   * @param Kp the proportional constant
   * @param setpoint the variable for the setpoint
   * @param measured the variable for the measurements
   * @param output the variable for the controller output
   */
  public PropCtrl(double Kp, DVar setpoint,
          DVar measured, DVar output){
    this.Kp = Kp;
    this.setpoint = setpoint;
    this.measured = measured;
    this.output = output;
  }

  /**
   * Executes this proportional controller action.
   */
  public void exec() {
    output.set((setpoint.get() - measured.get()) * Kp);
  }

  @Override
  public String toString() {
    return "PropCtrl [Kp=" + Kp + ", setpoint=" + setpoint + ", measured=" + measured + ", output=" + output + "]";
  }

}

