package at.jku.ssw.fp.sect05_4.dsl;

/**
 * Class for building a test model for the level control system.
 * Is a simple simulation model of a tank.
 *
 * @author Herbert Praehofer
 *
 */
public class SystemUnderCtrl implements Action {

  /** The base area of the tank */
  private final double area;

  /** Losses of the tank */
  private final double losses;

  /** The variable for the current level of the tank */
  private final DVar level;

  /** The variable for the inflow to the tank */
  private final DVar flow;

  /** */
  private long lastTime;

  /**
   * Constructor setting up the tank
   * @param area the base area
   * @param losses the losses of the tank
   * @param level the level variable
   * @param inflow the inflow variable
   */
  public SystemUnderCtrl(double area, double losses, DVar level, DVar inflow) {
    super();
    this.area = area;
    this.losses = losses;
    this.level = level;
    this.flow = inflow;
    this.lastTime = System.currentTimeMillis();
  }

  /**
   * Executes the model.
   */
  public void exec() {
    long time = System.currentTimeMillis();
    long timeStep = time - lastTime;
    double change = (flow.get() / area) * timeStep / 1000 - (losses * level.get()) * timeStep / 1000;
    System.out.format("Time step: %d, change %6.4f%n", timeStep, change);
    level.set(level.get() + change);
    lastTime = time;
  }

  @Override
  public String toString() {
    return String.format("SystemUnderCtrl: level= %6.2f, flow=%6.2f", level.get(), flow.get());
  }

}
