package at.jku.ssw.fp.sect06_2;

import static at.jku.ssw.fp.sect06_2.Monoids.intPlusMonoid;
import static at.jku.ssw.fp.sect06_2.Monoids.optionalMonoid;
import static at.jku.ssw.fp.sect06_2.Monoids.setMonoid;
import static at.jku.ssw.fp.sect06_2.Monoids.stringMonoid;

import java.util.List;
import java.util.Optional;
import java.util.Set;

/**
 * Demo showing reducible iterables. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_ReducibleIterable {

  @SuppressWarnings("unused")
  public static void main(String[] args) {

    List<String> list = List.of("Franz", "Fritz", "Berta", "Xaver"); 
    Reducible<String> reducible = Reducible.from(list); 
    
    String concatenated = reducible.reduce(stringMonoid);
    int totalLength = reducible.reduceMap(n -> n.length(), 
                        intPlusMonoid);

    // Concatenating names 

    String a = Reducible.from(list).reduce(Monoids.stringMonoid);
    System.out.println(a);

    // Sum of length of names 

    int ll = Reducible.from(list).reduceMap(n -> n.length(), Monoids.intPlusMonoid);
    System.out.println(ll);

    // Concatenating and sum from set of names 

    Set<String> set = Set.of("Franz", "Fritz", "Berta", "Hubert");

    String b = Reducible.from(set).reduce(Monoids.stringMonoid);
    System.out.println(b);

    int ls = Reducible.from(set).reduceMap(n -> n.length(), Monoids.intPlusMonoid);
    System.out.println(ls);

    // Union of sets 

    Set<Set<Integer>> listOfSets = Set.of(Set.of(1, 2, 3), Set.of(2, 4), Set.of(1, 3, 4));
    Set<Integer> union = Reducible.from(listOfSets).reduce(setMonoid());
    System.out.println(union);

    // Sum of Optional values
    
    List<Optional<Integer>> listOfOptInts = List.of(Optional.of(200), Optional.of(300), Optional.empty());
    Optional<Integer> optSum = Reducible.from(listOfOptInts).reduce(optionalMonoid(intPlusMonoid));
    System.out.println(optSum);

  }

}
