package at.jku.ssw.fp.sect06_2;

import static at.jku.ssw.fp.sect06_2.Course.Algo;
import static at.jku.ssw.fp.sect06_2.Course.BS;
import static at.jku.ssw.fp.sect06_2.Course.FM;
import static at.jku.ssw.fp.sect06_2.Course.GdP;
import static at.jku.ssw.fp.sect06_2.Course.POPL;
import static at.jku.ssw.fp.sect06_2.Course.SW2;
import static at.jku.ssw.fp.sect06_2.Monoids.intPlusMonoid;
import static at.jku.ssw.fp.sect06_2.Monoids.setMonoid;
import static at.jku.ssw.fp.sect06_2.Student.Franz;
import static at.jku.ssw.fp.sect06_2.Student.Lois;
import static at.jku.ssw.fp.sect06_2.Student.Sepp;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Demo showing reduction of a list of maps.
 *
 * @author Herbert Praehofer
 */
public class Demo4_ReduceMap {

  public static void main(String[] args) {

    // Count the occurrences of letters in texts

    List<Map<Character, Integer>> listOfOccurrences = List.of(Map.of('a', 224, 'e', 321, 'i', 122, 'o', 103, 'u', 93),
        Map.of('a', 143, 'e', 210, 'i', 199, 'o', 161, 'u', 88),
        Map.of('a', 212, 'e', 309, 'i', 133, 'o', 165, 'u', 121));
    Map<Character, Integer> occurrences = Reducible.from(listOfOccurrences).reduce(mapMonoid(intPlusMonoid));
    System.out.println(occurrences);

    // Combination of lines with letters

    List<Map<String, Set<Integer>>> listOfIndices = List.of(Map.of("a", Set.of(1, 3, 7), "b", Set.of(2, 3, 5)),
        Map.of("b", Set.of(3, 4, 7), "c", Set.of(2, 5), "d", Set.of(1, 3)),
        Map.of("a", Set.of(2, 6), "b", Set.of(4, 6), "d", Set.of(2)));

    Map<String, Set<Integer>> indicesCombined = Reducible.from(listOfIndices).reduce(mapMonoid(setMonoid()));
    System.out.println(indicesCombined);

    // Combination of courses per semester to courses taken alltogether

    List<Map<Student, Set<Course>>> listOfCoursesTakenInSemesters = List.of(
        Map.of(Franz, Set.of(GdP), Sepp, Set.of(GdP, SW2), Lois, Set.of(Algo)),
        Map.of(Franz, Set.of(BS, Algo), Sepp, Set.of(Algo), Lois, Set.of(BS)),
        Map.of(Franz, Set.of(FM), Sepp, Set.of(BS), Lois, Set.of(FM, POPL)));

    Monoid<Map<Student, Set<Course>>> courseMapMonoid =
        mapMonoid(setMonoid());

    Map<Student, Set<Course>> allCoursesTaken =
        Reducible.from(listOfCoursesTakenInSemesters)
                 .reduce(courseMapMonoid);

    System.out.println(allCoursesTaken);

  }

  static <K, V> Monoid<Map<K, V>> mapMonoid(Monoid<V> vMonoid) {
    return
        Monoid.of(Map.of(),
          (Map<K, V> a, Map<K, V> b) -> {
             Map<K, V> result = new HashMap<K, V>(a);
             for (Map.Entry<K, V> entry : b.entrySet()) {
               V aVal = result.get(entry.getKey());
               if (aVal == null)
                 result.put(entry.getKey(), entry.getValue());
               else
                 result.put(entry.getKey(),
                            vMonoid.combine(entry.getValue(), aVal));
             }
             return result;
          }
        );
    }


}
