package at.jku.ssw.fp.sect06_3;

import static at.jku.ssw.fp.sect06_3.Parser.alt;
import static at.jku.ssw.fp.sect06_3.Parser.anyToken;
import static at.jku.ssw.fp.sect06_3.Parser.mult;
import static at.jku.ssw.fp.sect06_3.Parser.token;

import at.jku.ssw.fp.coll.FList;
import at.jku.ssw.fp.util.Pair;

/**
 * Demo showing monad operations of combinator parsers. 
 * 
 * @author Herbert Praehofer
 */
public class Demo2_MonadicParser {

  static Parser<String> notTkn = token("!");
  static Parser<String> orTkn = token("||");
  static Parser<String> andTkn = token("&&");

  @SuppressWarnings("unused")
  public static void main(String[] args) {

    FList<String> tokens = FList.of("true false".split(" "));
    Result<? extends String> result = anyToken.apply(tokens);
    System.out.println("anyToken(true false) --> " + result);

    Parser<String> trueToken = anyToken.filter(t -> t.equals("true"));
    Result<? extends String> result2 = trueToken.apply(tokens);
    System.out.println("anyToken(true false).filter(t -> t.equals(true) --> " + result2);
    Result<? extends String> result3 = trueToken.apply(result2.rest);
    System.out.println("anyToken(true false).filter(t -> t.equals(true) --> " + result3);

    Parser<Boolean> trueParser = token("true").map(t -> true);
    Parser<Boolean> falseParser = token("false").map(t -> false);

    Parser<Pair<Boolean, Boolean>> trueFalseParser = trueParser.seq(falseParser);

    Result<Pair<Boolean, Boolean>> result4 = trueFalseParser.apply(tokens);

    Parser<Boolean> booleanParser = alt(trueParser, falseParser);

    Parser<FList<Boolean>> multTrueParser = mult(trueParser);
    tokens = FList.of("true true true false".split(" "));
    Result<FList<Boolean>> multTrues = multTrueParser.apply(tokens);
    System.out.println(multTrues);

  }

}
