package at.jku.ssw.fp.sect06_3_4;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.when;

import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

import at.jku.ssw.fp.expr.Expr;
import at.jku.ssw.fp.expr.Expr.And;
import at.jku.ssw.fp.expr.Expr.Lit;
import at.jku.ssw.fp.expr.Expr.Not;
import at.jku.ssw.fp.expr.Expr.Or;
import at.jku.ssw.fp.expr.Expr.Var;

/**
 * Demo of evaluation of expressions in an imperative way with exceptions. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_ExprEvalSequentialExcpt {

    static boolean eval(Expr expr, Map<String, Boolean> bds) {
        return when(expr,
            is(Lit.class,
                        lit -> lit.value),
            is(Var.class,
                        vbl -> {
                            Boolean vblVal = bds.get(vbl.name);
                            if (vblVal == null) throw new NoSuchElementException("No binding for variable " + vbl);
                            return vblVal;
                        }),
            is(Not.class,
                        not -> {
                            boolean subVal = eval(not.sub, bds);
                            return ! subVal;
                        }),
            is(And.class,
                        and -> {
                            boolean leftVal = eval(and.left, bds);
                            boolean rightVal = eval(and.right, bds);
                            return leftVal && rightVal;
                        }),
            is(Or.class,
                        or -> {
                            boolean leftVal = eval(or.left, bds);
                            boolean rightVal = eval(or.right, bds);
                            return leftVal || rightVal;
                        })
        );
    }

    public static void main(String[] args) {
        Expr expr =
                new Or(
                        new Var("x"),
                        new Var("y")
                );

        Map<String, Boolean> bds = new HashMap<>();
        bds.put("x", true);
        //bds.put("y", false);

        Boolean result = eval(expr, bds);
        System.out.format("%s = %s", expr.toString(), result.toString());
    }

}
