package at.jku.ssw.fp.sect06_3_4;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.when;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import at.jku.ssw.fp.expr.Expr;
import at.jku.ssw.fp.expr.Expr.And;
import at.jku.ssw.fp.expr.Expr.Lit;
import at.jku.ssw.fp.expr.Expr.Not;
import at.jku.ssw.fp.expr.Expr.Or;
import at.jku.ssw.fp.expr.Expr.Var;

/**
 * Demo of evaluation of expressions with Monad Optional.
 * 
 * @author Herbert Praehofer
 */
public class Demo3_ExprEvalOptional {

     static Optional<Boolean> eval (Expr expr, Map<String, Boolean> bds) {
       return 
           when(expr,
             is(Lit.class, lit -> Optional.of(lit.value)),
             is(Var.class, 
                vbl -> Optional.ofNullable(bds.get(vbl.name))),
             is(And.class, 
                and -> eval(and.left, bds)
                       .flatMap(leftVal -> eval(and.right, bds)
                         .flatMap(rightVal -> 
                           Optional.of(leftVal && rightVal)))),

            is(Not.class, not ->
                eval(not.sub, bds)
                        .flatMap(subVal -> Optional.of(! subVal))),
             is(Or.class, 
                 or ->  eval(or.left, bds)
                        .flatMap(leftVal -> eval(or.right, bds)
                                .flatMap(rightVal -> Optional.of(leftVal || rightVal))))
        );
    }

    public static void main(String[] args) {
        Expr expr =
                new Or(
                        new Var("x"),
                        new Var("y")
                );

        Map<String, Boolean> bds = new HashMap<>();
        bds.put("x", true);
        bds.put("y", false);

        Optional<Boolean> result = eval(expr, bds);
        System.out.format("%s = %s", expr.toString(), result.toString());
    }

}
