package at.jku.ssw.fp.sect06_3_4;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.when;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import at.jku.ssw.fp.coll.FSet;
import at.jku.ssw.fp.expr.Expr;
import at.jku.ssw.fp.expr.Expr.And;
import at.jku.ssw.fp.expr.Expr.Lit;
import at.jku.ssw.fp.expr.Expr.Not;
import at.jku.ssw.fp.expr.Expr.Or;
import at.jku.ssw.fp.expr.Expr.Var;

/**
 * Demo of evaluation of expressions with Monad FSet.
 * 
 * @author Herbert Praehofer
 */
public class Demo4_ExprEvalFSet {

  static FSet<Boolean> eval(Expr expr, Map<String, Set<Boolean>> bds) {
    return when(expr,
      is(Lit.class, lit -> FSet.of(lit.value)),
      is(Var.class, vbl -> FSet.of(bds.get(vbl.name))),
      is(And.class, 
        and ->  eval(and.left, bds)
         .flatMap(leftVal -> eval(and.right, bds)
           .flatMap(rightVal -> 
             FSet.of(leftVal && rightVal)))),
            is(Not.class, not ->
                    eval(not.sub, bds)
                            .flatMap(subVal -> FSet.of(! subVal))),
            is(Or.class, or ->  eval(or.left, bds)
                    .flatMap(leftVal -> eval(or.right, bds)
                            .flatMap(rightVal -> FSet.of(leftVal || rightVal))))
        );
    }

    public static void main(String[] args) {
        Expr expr =
                new And(
                        new Var("x"),
                        new Var("y")
                );

        Map<String, Set<Boolean>> bds = new HashMap<>();
        Set<Boolean> xVal = Set.of(true, false);
        Set<Boolean> yVal = Set.of(true);
        bds.put("x", xVal);
        bds.put("y", yVal);

        FSet<Boolean> result = eval(expr, bds);
        System.out.format("%s = %s%n", expr.toString(), result.toString());

        xVal = Set.of();
        bds.put("x", xVal);
        FSet<Boolean> result2 = eval(expr, bds);
        System.out.format("%s = %s%n", expr.toString(), result2.toString());
    }



}
