package at.jku.ssw.fp.sect06_3_4;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.when;

import java.util.HashMap;
import java.util.Map;

import at.jku.ssw.fp.expr.Expr;
import at.jku.ssw.fp.expr.Expr.And;
import at.jku.ssw.fp.expr.Expr.Lit;
import at.jku.ssw.fp.expr.Expr.Not;
import at.jku.ssw.fp.expr.Expr.Or;
import at.jku.ssw.fp.expr.Expr.Var;

/**
 * Demo of evaluation of expressions with Monad Logged.
 * 
 * @author Herbert Praehofer
 */
public class Demo5_ExprEvalLogged {

    static Logged<Boolean> eval(Expr expr, Map<String, Boolean> bds) {
      return 
          when(expr,
            is(Lit.class, 
               lit -> Logged.of(lit.value, lit.toString())),
            is(Var.class, 
               vbl -> Logged.of(bds.get(vbl.name), vbl.toString())),
            is(And.class, 
               and ->  eval(and.left, bds)
                   .flatMap(leftVal -> eval(and.right, bds)
                     .flatMap(rightVal -> 
                      Logged.of(leftVal && rightVal, 
                          and.toString())))),
            is(Not.class, 
               not -> eval(not.sub, bds)
                            .flatMap(subVal -> Logged.of(! subVal, not.toString()))),
             is(Or.class, 
                or ->  eval(or.left, bds)
                          .flatMap(leftVal -> eval(or.right, bds)
                              .flatMap(rightVal -> Logged.of(leftVal || rightVal, or.toString()))))
        );
    }

    public static void main(String[] args) {
        Expr expr =
                new And(
                        new Var("x"),
                        new Var("y")
                );

        Map<String, Boolean> bds = new HashMap<>();
        bds.put("x", true);
        bds.put("y", false);

        Logged<Boolean> result = eval(expr, bds);
        System.out.format("%s = %s", expr.toString(), result.toString());
    }


}
