package at.jku.ssw.fp.sect06_3_4;

import static at.jku.ssw.fp.util.When.is;
import static at.jku.ssw.fp.util.When.when;

import java.util.HashMap;
import java.util.Map;

import at.jku.ssw.fp.expr.Expr;
import at.jku.ssw.fp.expr.Expr.And;
import at.jku.ssw.fp.expr.Expr.Lit;
import at.jku.ssw.fp.expr.Expr.Not;
import at.jku.ssw.fp.expr.Expr.Or;
import at.jku.ssw.fp.expr.Expr.Var;

/**
 * Demo of evaluation of expressions with Monad Reader.
 * 
 * @author Herbert Praehofer
 */
public class Demo6_ExprEvalReader {

     static Reader<Map<String, Boolean>, Boolean> eval(Expr expr) {
       return when(expr,
           is(Lit.class, lit -> Reader.of(ctx -> lit.value)),
           is(Var.class, 
              vbl -> Reader.of(ctx -> ctx.get(vbl.name))),
           is(And.class, 
             and -> eval(and.left)
                      .flatMap(leftVal -> eval(and.right)
                        .flatMap(rightVal -> 
                          Reader.of(ctx -> leftVal && rightVal)))),
           is(Not.class, 
              not -> eval(not.sub)
                        .flatMap(subVal -> Reader.of(e -> ! subVal))),
           is(Or.class, 
              or ->  eval(or.left)
                        .flatMap(leftVal -> eval(or.right)
                            .flatMap(rightVal -> Reader.of(ctx -> leftVal || rightVal))))
        );
   }

    public static void main(String[] args) {

        Expr expr =
                new Or(
                        new Not(new Var("x")),
                        new Var("y")
                );

        Map<String, Boolean> bds = new HashMap<>();
        bds.put("x", true);
        bds.put("y", false);

        Reader<Map<String, Boolean>, Boolean> reader = eval(expr);
        boolean result = reader.apply(bds);
        System.out.format("%s = %s", expr.toString(), result);
    }

}
