package at.jku.ssw.fp.sect06_3_4;

import java.util.function.Function;

/**
 * Class representing the identity monad.
 *
 * @author Herbert Praehofer
 *
 * @param <T> the type of values
 */
public class Id<T> {

  /**
   * Creates an Id monad with a wrapped value.
   *
   * @param <T>  the type of the value
   * @param elem the wrapped value
   * @return the monad value with the wrapped value
   */
  public static <T> Id<T> of(T elem) {
    return new Id<T>(elem);
  }

  /** The wrapped value. */
  public final T elem;

  /**
   * Constructor wrapping a value.
   *
   * @param elem the wrapped value
   */
  private Id(T elem) {
    this.elem = elem;
  }

  /**
   * Monad operator.
   *
   * @param <R>    the type of the wrapped result object.
   * @param mapper the mapping function
   * @return the wrapped result value
   */
  @SuppressWarnings("unchecked")
  public <R> Id<R> flatMap(Function<? super T, ? extends Id<? extends R>> mapper) {
    return (Id<R>) mapper.apply(elem);
  }
}
