package at.jku.ssw.fp.sect07_3;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Optional;
import java.util.Random;
import java.util.Set;
import java.util.SplittableRandom;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import at.jku.ssw.fp.sect07_1.Article;

/**
 * Demo of operations for building streams.
 * 
 * @author Herbert Praehofer 
 *
 */
public class Demo1_BuilderOperations {

  @SuppressWarnings("unused")
  public static void main(String[] args) {

    Article[] articleArray = new Article[] { new Article("A", 700.0), new Article("B", 1500), new Article("C", 1200),
        new Article("D", 200), new Article("E", 100), new Article("F", 800) };

    Stream<Article> articleStream = Arrays.stream(articleArray);
    articleStream.forEach(System.out::println);

    IntStream intStream = Arrays.stream(new int[] { 1, 2, 3, 4, 5, 6, 7 });
    System.out.println();
    intStream.forEach(System.out::println);

    Stream<String> words1 = 
        Stream.of("Funktionale", "Programmierung");

    Stream<String> words = 
        Stream.concat(words1, Stream.of("in", "Java"));

    System.out.println();
    words.forEach(System.out::println);

    IntStream range0_99 = IntStream.range(0, 100);
    System.out.println();
    range0_99.forEach(System.out::println);

    IntStream range0_100 = IntStream.rangeClosed(0, 100);
    System.out.println();
    range0_100.forEach(System.out::println);

    System.out.println();
    try (Stream<String> lines = Files.lines(Paths.get("files/text.txt"))) {
      lines.forEach(System.out::println);
    } catch (IOException e) {
      e.printStackTrace();
    }

    System.out.println();
    try (Stream<String> lines = Files.lines(Paths.get("files/text.txt"))) {
      Optional<String> firstLineWithJava = 
          lines.filter(line -> line.contains("Java")).findFirst();
      System.out.println(firstLineWithJava);
    } catch (IOException e) {
      e.printStackTrace();
    }

    System.out.println();
    try (BufferedReader reader = Files.newBufferedReader(Paths.get("files/text.txt"))) {
      Stream<String> lines = reader.lines();
      Optional<String> firstLineWithJava = lines.filter(line -> line.contains("Java")).findFirst();
      System.out.println(firstLineWithJava);
    } catch (IOException e) {
      e.printStackTrace();
    }

    String text = "This is a long text with many characters";
    Set<Character> charactersInText = 
        text.chars()
           .map(c -> Character.toLowerCase(c))
           .filter(c -> Character.isLetter(c))
           .mapToObj(c -> Character.valueOf((char) c))
           .collect(Collectors.toSet());

    System.out.println(charactersInText);

    System.out.println();
    Random rand = new Random(7);
    IntStream ints1 = rand.ints();
    IntStream ints2 = rand.ints(0, 10);
    IntStream ints3 = rand.ints(20L);
    IntStream ints4 = rand.ints(20L, 0, 10);

    ints3.forEach(System.out::println);
    System.out.println();
    ints4.forEach(System.out::println);
    System.out.println();

    System.out.println();
    SplittableRandom splitableRandom = new SplittableRandom(7);
    IntStream ints5 = splitableRandom.ints();
    IntStream ints6 = splitableRandom.ints(0, 10);
    IntStream ints7 = splitableRandom.ints(20L);
    IntStream ints8 = splitableRandom.ints(20L, 0, 10);

    ints7.forEach(System.out::println);
    System.out.println();
    ints8.forEach(System.out::println);
    System.out.println();

  }

}
