package at.jku.ssw.fp.sect07_3;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import at.jku.ssw.fp.sect07_1.Article;

/**
 * Demo of intermediate stream operations.
 * 
 * @author Herbert Praehofer 
 *
 */
public class Demo2_IntermediateOperations {

  public static void main(String[] args) {

    // flatMap

    try (Stream<String> wordStream = 
          Files.lines(Paths.get("files/text.txt"))
             .flatMap(line -> Arrays.stream(line.split(" ")))) {
      System.out.println();
      wordStream.forEach(System.out::println);

    } catch (IOException e) {
      e.printStackTrace();
    }

    // flatMap and sorted

    try (Stream<String> sortedWordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" "))).sorted()) {
      System.out.println();
      sortedWordStream.forEach(System.out::println);

    } catch (IOException e) {
      e.printStackTrace();
    }

    // peek

    List<Article> articles = List.of(new Article("A", 700), new Article("B", 1500), new Article("C", 1200),
        new Article("D", 200), new Article("E", 100), new Article("F", 800));

    System.out.println();
    List<String> highPriceArticleNames = 
        articles.stream()
            .filter(a -> a.price > 1000.0)
            .peek(a -> System.out.println("Before sort: " + a))
            .sorted(Comparator.comparingDouble(a -> a.price))
            .peek(a -> System.out.println("After sort: " + a))
            .map(a -> a.name)
            .collect(Collectors.toList());

    System.out.println();
    System.out.println(highPriceArticleNames);

  }

}
