package at.jku.ssw.fp.sect07_3;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.OptionalInt;
import java.util.Random;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import at.jku.ssw.fp.util.LongRef;

/**
 * Demo of terminal stream operations.
 * 
 * @author Herbert Praehofer 
 *
 */
public class Demo3_TerminalOperations {

  public static void main(String[] args) {

    Random rand = new Random(7);

    IntStream numbers = IntStream.generate(() -> rand.nextInt(100)).limit(20);
    OptionalInt max = numbers.reduce((a, b) -> Integer.max(a, b));
    System.out.println(max);

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      List<String> wordList = wordStream.reduce(new ArrayList<String>(), (list, elem) -> {
        list.add(elem);
        return list;
      }, (list1, list2) -> {
        list1.addAll(list2);
        return list1;
      });
      System.out.println();
      System.out.println(wordList);

    } catch (IOException e) {
      e.printStackTrace();
    }

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      List<String> wordList = 
          wordStream.collect(  
                       ArrayList::new, 
                       ArrayList::add, 
                       ArrayList::addAll
                     ); 

      System.out.println();
      System.out.println(wordList);

    } catch (IOException e) {
      e.printStackTrace();
    }

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      List<String> wordList = wordStream.collect(ArrayList::new, ArrayList::add, ArrayList::addAll);
      System.out.println();
      System.out.println(wordList);

    } catch (IOException e) {
      e.printStackTrace();
    }

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      String sentence = 
          wordStream.collect(
                       StringBuilder::new, 
                       (b, w) -> { b.append(w); b.append(" "); }, 
                       (b1, b2) -> b1.append(b2)
                     ).toString();

      System.out.println();
      System.out.println(sentence);
    } catch (IOException e) {
      e.printStackTrace();
    }

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      LongRef nWords = wordStream.collect(() -> LongRef.of(0), (n, w) -> n.value += 1,
          (n1, n2) -> n1.value += n2.value);
      System.out.println();
      System.out.println(nWords);
    } catch (IOException e) {
      e.printStackTrace();
    }

  }

  public static <T> Optional<T> min(Stream<T> stream, Comparator<? super T> comparator) {
    return stream.reduce((a, b) -> comparator.compare(a, b) <= 0 ? a : b);
  }

}
