package at.jku.ssw.fp.sect07_4;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Demo of collectors.
 * 
 * @author Herbert Praehofer 
 *
 */
public class Demo1_Collector {

  public static void main(String[] args) {

    // collect ArrayList
    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      List<String> wordList = 
          wordStream.collect(
                 Collector.of(
                         ArrayList<String>::new,
                         ArrayList::add, 
                         (list1, list2) -> { 
                           list1.addAll(list2); 
                           return list1;
                         }
                       )
                     );
      
      System.out.println();
      System.out.println(wordList);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // toList

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      List<String> wordList = wordStream.collect(Collectors.toList());
      
      System.out.println();
      System.out.println(wordList);

    } catch (IOException e) {
      e.printStackTrace();
    }

    // toCollection

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Collection<String> sortedWordSet = 
          wordStream.collect(
                       Collectors.toCollection(TreeSet<String>::new));

      System.out.println();
      System.out.println(sortedWordSet);

    } catch (IOException e) {
      e.printStackTrace();
    }

    // toMap

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<String, Integer> wordCount = wordStream.collect(
          Collectors.toMap(w -> w, w -> 1, (n, one) -> n + one));

      System.out.println();
      System.out.println(wordCount);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // joining

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      String wordString  = 
          wordStream.collect(Collectors.joining(", ", "{", "}"));

      System.out.println();
      System.out.println(wordString);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // groupingBy

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<Character, List<String>> wordGroups = wordStream.collect(
          Collectors.groupingBy(
                          w -> Character.toLowerCase(w.charAt(0))));

      System.out.println();
      System.out.println(wordGroups);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // partitioningBy

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<Boolean, List<String>> upperLowerWords = wordStream.collect(
          Collectors.partitioningBy(
                          w -> Character.isUpperCase(w.charAt(0))));


      System.out.println();
      System.out.println(upperLowerWords);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // groupingBy mit SortedSets

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<Character, SortedSet<String>> wordGroups = 
          wordStream.collect(Collectors.groupingBy(
                                 w -> w.charAt(0),  
                                 Collectors.toCollection(TreeSet::new))
                            );
      System.out.println();
      System.out.println(wordGroups);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // groupingBy mit Anzahl

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<Character, Long> nWordsPerGroup = 
          wordStream.collect(Collectors.groupingBy(
                                 w -> w.charAt(0), 
                                 Collectors.counting())
                              );

      System.out.println();
      System.out.println(nWordsPerGroup);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // groupingBy mit Summe

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<Character, Integer> sumWordLengthPerGroup = 
          wordStream.collect(Collectors.groupingBy(
                                w -> w.charAt(0), 
                                Collectors.summingInt(w -> w.length()))
                             );

      System.out.println();
      System.out.println(sumWordLengthPerGroup);
    } catch (IOException e) {
      e.printStackTrace();
    }

    // groupingBy mit Durchschnitt

    try (Stream<String> wordStream = Files.lines(Paths.get("files/text.txt"))
        .flatMap(line -> Arrays.stream(line.split(" ")))) {
      
      Map<Character, Integer> sumWordLengthPerGroup = wordStream
          .collect(Collectors.groupingBy(word -> word.charAt(0), Collectors.summingInt(w -> w.length())));

      System.out.println();
      System.out.println(sumWordLengthPerGroup);
    } catch (IOException e) {
      e.printStackTrace();
    }

  }

}
