package at.jku.ssw.fp.sect07_5.index;

import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.mapping;
import static java.util.stream.Collectors.toList;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import at.jku.ssw.fp.util.IntRef;

/**
 * Demo showing the computation of a word index for a text.
 *
 * @author Herbert Praehofer
 *
 */
public class WordIndex {

  private static final String TEXT_FILE = "files/faust_1.txt";

  public static void main(String[] args) throws IOException {


    try (Stream<String> lines = Files.lines(Paths.get(TEXT_FILE))) {
      List<String> wordLst = lines.flatMap(line -> Arrays.stream(line.split("[ .,;?!.:()]")))
          .map(word -> word.toLowerCase()).filter(word -> word.length() != 0)
          .filter(word -> Character.isLetter(word.charAt(0))).sorted().distinct().collect(toList());

      wordLst.stream().forEach(System.out::println);
    }

    try (Stream<String> lines = Files.lines(Paths.get(TEXT_FILE))) {

      final IntRef i = IntRef.of(0);
      Map<String, SortedSet<Integer>> index =
          lines
            .map(line -> {
              i.value++;
              return Numbered.of(i.value, line);
            })
            .flatMap(numLine -> {
              String[] words = numLine.value.split("[ .,;?!.:()]");
              return Arrays.stream(words)
                  .map(word -> word.trim().toLowerCase())
                  .filter(word -> word.length() != 0 &&
                             Character.isLetter(word.charAt(0)))
                  .map(word -> Numbered.of(numLine.n, word));
            })
            .collect(
                Collectors.groupingBy(
                   numWord -> numWord.value,
                   TreeMap<String, SortedSet<Integer>>::new,
                   mapping(numWord -> numWord.n,
                       Collectors.toCollection(TreeSet<Integer>::new))));


      System.out.println("\nIndex: ");
      index.entrySet().forEach(e -> {
        System.out.format("%s: %s%n", e.getKey(), e.getValue().stream().map(Object::toString).collect(joining(", ")));
      });

    }

  }

}
