package at.jku.ssw.fp.sect07_5.results;

import java.util.Arrays;
import java.util.function.IntPredicate;

/**
 * Class representing the results of a student.
 *
 * @author Herbert Praehofer
 */
public class Results {

  /** Number of assignments */
  private static final int N = 10;

  /** The student id */
  private final String id;

  /** The name of the student */
  private final String name;

  /** The array of scores for the different assignments */
  private final int[] scores;

  /**
   * Constructor initializing id, name and scores.
   *
   * @param id the student id
   * @param name the student name
   * @param scores the scores for the different assignments
   */
  public Results(String id, String name, int... scores) {
    this.id = id;
    this.name = name;
    this.scores = new int[N];
    for (int i = 0; i < N; i++) {
      this.scores[i] = -1;
    }
    for (int i = 0; i < scores.length; i++) {
      setScore(i + 1, scores[i]);
    }
  }

  /**
   * Returns the id of the student.
   *
   * @return the id of the student
   */
  public String getStdId() {
    return id;
  }

  /**
   * Returns the name of the student.
   *
   * @return the name of the student
   */
  public String getName() {
    return name;
  }

  /**
   * Returns the score of an assignment.
   *
   * @param assignment the number of the assignment.
   * @return the score of the assignment
   */
  public int getScore(int assignment) {
    assert assignment >= 1 && assignment <= 10;
    return scores[assignment - 1];
  }

  /**
   * Returns the array of the scores for the {@link N} assignments.
   *
   * @return the array of the scores
   */
  public int[] getScores() {
    return scores;
  }

  /**
   * Sets the score of an assignment.
   *
   * @param assignment the number of the assignment
   * @param score the score for the assignment
   */
  public void setScore(int assignment, int score) {
    assert assignment >= 1 && assignment <= 10;
    assert score >= 0 && assignment <= 10;
    scores[assignment - 1] = score;
  }

  /**
   * Sets the scores for the {@link N} assignments.
   *
   * @param scores the array with the scores
   */
  public void setScores(int... scores) {
    assert (scores.length == 10);
    for (int i = 1; i <= N; i++) {
      setScore(i, scores[i - 1]);
    }
  }

  /**
   * Counts the scores fulfilling a given predicate.
   *
   * @param pred the predicate to test the scores
   * @return the number of scores fulfilling the given predicate
   */
  public int countScores(IntPredicate pred) {
    return (int)Arrays.stream(getScores()).filter(pred).count();
  }

  @Override
  public String toString() {
    return "Results [id=" + id + ", name=" + name + ", scores=" + Arrays.toString(scores) + "]";
  }

}
