package at.jku.ssw.fp.sect07_6;

import java.util.HashSet;
import java.util.List;
import java.util.OptionalInt;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

/**
 * Demos showing does and don'ts of streams.
 *
 * @author Herbert Praehofer
 *
 */
public class Demo1_Hints {

  public static void main(String[] args) {

    // iterate once

    List<String> wordList = List.of("Java", "Funktionale", "Programmierung", "in", "Java");

    Stream<String> wordStream = wordList.stream();
    System.out.println("\nwordStream 1:");
    wordStream.forEach(System.out::println);
    wordStream = wordList.stream(); // neues Stream-Objekt notwendig !
    System.out.println("\nwordStream 2:");
    wordStream.forEach(System.out::println);

    // limit infinite Streams

    IntStream posInts = IntStream.iterate(0, i -> i + 1);
    System.out.println("\nlimit(100): ");
    posInts.limit(100).forEach(System.out::println);

    posInts = IntStream.iterate(0, i -> i + 1);
    OptionalInt firstPrime = posInts.skip(100).filter(x -> isPrime(x)).findFirst();
    System.out.println("\nfirstPrime: ");
    System.out.println(firstPrime);

    // stateless vs. stateful

    System.out.println("\nstateless: ");

    Stream<String> stream = Stream.of("C", "B", "A");
    stream.map(s -> s.toLowerCase())
          .peek(s -> System.out.format("First map: %s%n", s))
          .map(s -> s.toUpperCase())
          .peek(s -> System.out.format("Second map: %s%n", s))
          .collect(Collectors.toList());

    System.out.println("\nstateful: ");

    stream = Stream.of("C", "B", "A");
    stream.map(s -> s.toLowerCase())
          .peek(s -> System.out.format("First map: %s%n", s))
          .sorted()
          .map(s -> s.toUpperCase())
          .peek(s -> System.out.format("Second map: %s%n", s))
          .collect(Collectors.toList());


    // operation order: limit and skip

    System.out.println("\nskip(2).limit(5): ");
    IntStream.iterate(1, i -> i + 1).skip(2).limit(5).forEach(System.out::println);

    System.out.println("\nlimit(5).skip(2): ");
    IntStream.iterate(1, i -> i + 1).limit(5).skip(2).forEach(System.out::println);

    // operation order: limit and skip

    System.out.println("\nfilter(  ).limit(5):");
    IntStream.iterate(1, i -> i + 1).filter(i -> i % 2 == 0).limit(5).forEach(System.out::println);

    System.out.println("\nlimit(5).filter(  ): ");
    IntStream.iterate(1, i -> i + 1).limit(5).filter(i -> i % 2 == 0).forEach(System.out::println);

    // operation order: limit and sorted

    System.out.println("\nsorted().limit(3):");
    List<Integer> someNumbers = List.of(3, 5, 2, 8, 9, 7, 1, 6);
    someNumbers.stream().sorted().limit(3).forEach(System.out::println);

    System.out.println("\nlimit(3).sorted():");
    someNumbers.stream().limit(3).sorted().forEach(System.out::println);

    // Combination: sorted and TreeSet

    SortedSet<String> sortedWords =
        wordList.stream()
                .sorted()         // sorted ist redundant
                .collect(Collectors.toCollection(TreeSet<String>::new));
    System.out.println("\nsorted and toCollection(TreeSet<String>::new):");
    System.out.println(sortedWords);

    Set<String> unsortedWords =
        wordList.stream()
                .sorted()         // sorted hat keine Wirkung
                .collect(Collectors.toCollection(HashSet<String>::new));
    System.out.println("\nsorted and toCollection(HashSet<String>::new):");
    System.out.println(unsortedWords);

    Set<String> distinctWords =
        wordList.stream()
                  .distinct()    // distinct ist redundant
                  .collect(Collectors.toSet());
    System.out.println("\ndistinct and toSet:");
    System.out.println(distinctWords);

  }

  private static boolean isPrime(long n) {
    for (int i = 2; i <= n / 2; i++) {
      if (n % i == 0)
        return false;
    }
    return true;
  }

}
