package at.jku.ssw.fp.sect07_7;

import static at.jku.ssw.fp.sect07_7.StreamUtil.number;
import static java.util.stream.Collectors.groupingBy;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.mapping;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import at.jku.ssw.fp.sect07_5.index.Numbered;

/**
 * Demo showing the improvement of the computation of a word index for a text by
 * building pairs of numbers and elements.
 *
 * @author Herbert Praehofer
 *
 */
public class Demo_Numbered {

  private static final String TEXT_FILE = "files/faust_1.txt";

  public static void main(String[] args) {

    try (Stream<String> lines = Files.lines(Paths.get(TEXT_FILE))) {
      Stream<Numbered<String>> numberedLines = number(lines);

      Map<String, Set<Integer>> index = numberedLines.flatMap(il -> {
        String[] words = il.value.split("[ .,;?!.:()]");
        return Arrays.stream(words).map(w -> w.trim().toLowerCase()).filter(w -> w.length() != 0)
            .map(w -> Numbered.of(il.n, w));
      }).collect(groupingBy(iw -> iw.value, () -> new TreeMap<String, Set<Integer>>(),
          mapping(iw -> iw.n, Collectors.toCollection(TreeSet<Integer>::new))));

      System.out.println("\nIndex2: ");
      index.entrySet().forEach(e -> {
        System.out.format("%s: %s%n", e.getKey(), e.getValue().stream().map(Object::toString).collect(joining(", ")));
      });
    } catch (IOException e) {
      e.printStackTrace();
    }

  }

}
