package at.jku.ssw.fp.sect07_7;

import java.util.Spliterator;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import at.jku.ssw.fp.sect07_5.index.Numbered;
import at.jku.ssw.fp.util.Pair;
import at.jku.ssw.fp.util.Ref;

/**
 *
 * @author Herbert Praehofer
 *
 */
public class StreamUtil {

  /**
   * Creates a stream of numbered values from a stream of values.
   *
   * @param <A> type of the value
   * @param as the stream of values
   * @return the stream of the numbered values
   */
  public static <A extends Comparable<A>> Stream<Numbered<A>> number(Stream<A> as) {
    IntStream is = IntStream.iterate(1, x -> x + 1);
    Stream.Builder<Numbered<A>> builder = Stream.builder();
    Spliterator<A> asSpltr = as.spliterator();
    Spliterator.OfInt isSpltr = is.spliterator();
    asSpltr.forEachRemaining(a -> {
      isSpltr.tryAdvance((int i) -> {
        builder.accept(Numbered.of(i, a));
      });
    });
    return  builder.build();
  }

  /**
   * Creates a stream of pairs of values from two streams.
   *
   * @param <A> the type of values of the first stream
   * @param <B> the type of values of the second stream
   * @param as the first stream
   * @param bs the second stream
   * @return the stream of pairs of values from the two streams
   */
  public static <A, B> Stream<Pair<A, B>> zip(Stream<A> as, Stream<B> bs) {
    Spliterator<A> aSpltr = as.spliterator();
    Spliterator<B> bSpltr = bs.spliterator();
    Stream.Builder<Pair<A, B>> builder = Stream.builder();
    Ref<Boolean> aHasMore = Ref.of(false);
    Ref<Boolean> bHasMore = Ref.of(false);
    do {
      aHasMore.value = aSpltr.tryAdvance(a -> {
        bHasMore.value = bSpltr.tryAdvance(b -> {
          builder.accept(Pair.of(a, b));
        });
      });
    } while (aHasMore.value && bHasMore.value);
    return builder.build();
  }

  /**
   * Creates a stream of numbered values from a stream of values.
   * Is an alternative implementation of  {@link #number(Stream)} using {@link #zip(Stream, Stream)}.
   *
   * @param <A> type of the value
   * @param as the stream of values
   * @return the stream of the numbered values
   */
  public static <A> Stream<Pair<Integer, A>> number2(Stream<A> as) {
    return zip(Stream.iterate(1, x -> x + 1), as);
  }

}
