package at.jku.ssw.fp.sect08_3;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import at.jku.ssw.fp.util.IntRef;
import at.jku.ssw.fp.util.LongRef;

/**
 * Demos showing the effect of arbitrary side effects in parallel stream operations.
 *
 * @author Herbert Praehofer
 *
 */
public class Demo1_SideEffect {

  static volatile int max;

  public static void main(String[] args) {

    List<Integer> listOfIntegers = createIntList(100);
    LongRef sum1 = LongRef.of(1);
    listOfIntegers.stream().forEach(x -> sum1.value = sum1.value + x);
    System.out.println("Sum seqential = " + sum1);

    LongRef sum2 = LongRef.of(1);
    listOfIntegers.parallelStream()
                  .forEach(x -> sum2.value = sum2.value + x);
    System.out.println("Sum parallel = " + sum2);

    final List<Integer> coll = createIntList(100);
    IntRef max1 = IntRef.of(0);
    List<Integer> filteredSeq1 =
       coll.stream()
           .filter(x -> {
             if (x > max1.value) {
               max1.value = x;
               return true;
             } else {
               return false;
             }
           })
           .collect(Collectors.toList());
    System.out.println("Filtered sequentia = " + filteredSeq1);

    IntRef max2 = IntRef.of(0);
    List<Integer> filteredSeq2 =
       coll.parallelStream()
           .filter(x -> {
             if (x > max2.value) {
               max2.value = x;
               return true;
             } else {
               return false;
             }
           })
           .collect(Collectors.toList());
    System.out.println("Filtered parallel = " + filteredSeq2);


  }

  private static List<Integer> createIntList(int n) {
    List<Integer> l = new ArrayList<>();
    for (int i = 0; i < n; i++) {
      l.add(i);
    }
    return l;
  }


}
