package at.jku.ssw.fp.sect08_3;

import static java.util.stream.Collectors.toList;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Demos showing parallel collect operations with normal and concurrent collections. 
 * 
 * @author Herbert Praehofer 
 *
 */
public class Demo2_ParallelCollect {

  private static final String TEXT_FILE = "files/faust_1.txt";

  public static void main(String[] args) throws IOException {

    //
    List<String> wordList = Files.lines(Paths.get(TEXT_FILE)).flatMap(line -> Arrays.stream(line.split("[ .,;?!.:()]")))
        .map(word -> word.toLowerCase()).filter(word -> word.length() != 0)
        .filter(word -> Character.isLetter(word.charAt(0))).sorted().collect(toList());

    Map<String, Integer> wordCountSeq = wordList.stream()
        .collect(Collectors.toMap(w -> w, w -> 1, (n, one) -> n + one));

    System.out.println("\nCounts seq: ");
    wordCountSeq.entrySet().forEach(System.out::println);

    Map<String, Integer> wordCountPar = 
        wordList
          .parallelStream()
          .collect(Collectors.toMap(w -> w, w -> 1, (n, x) -> n + x));

    System.out.println("\nCounts par: ");
    wordCountPar.entrySet().forEach(System.out::println);

    Map<String, Integer> wordCountCon = 
        wordList
          .parallelStream()
          .collect(Collectors.toConcurrentMap(w -> w, 
                                              w -> 1, 
                                              (n, x) -> n + x));

    System.out.println("\nCounts conc: ");
    wordCountCon.entrySet().forEach(System.out::println);

  }

}
