package at.jku.ssw.fp.sect09_1;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;

/**
 * Demo showing the use of parallel streams for blocking calls. 
 * 
 * @author Herbert Praehofer
 *
 */
public class Demo1_ParallelStream {

  private static final String PARALLELISM = "3";
 
  static final Random RAND = new Random();

  public static void main(String[] args) {

    System.setProperty("java.util.concurrent.ForkJoinPool.common.parallelism", PARALLELISM);

    long start = System.currentTimeMillis();

    Airline[] airlines = Airline.AIRLINES; 
    LocalDate someDate = LocalDate.now().plusDays(7); 
    
    List<Flight> flights = 
        Arrays.stream(airlines)
          .parallel()
          .map(airline -> airline.findFlight("VIE", "JKF", someDate))
          .sorted(Comparator.comparingInt(flight -> flight.price))
          .collect(Collectors.toList());

     for (Flight flight : flights) {
      System.out.println(flight.toString());
    }

    System.out.format("Execution time: %d ms%n", System.currentTimeMillis() - start);
    
  }

}
