package at.jku.ssw.fp.sect09_1;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;

/**
 * Demo showing the use of parallel streams for blocking calls. 
 * Uses an increased parallelism in the fork-join pool.  
 * 
 * @author Herbert Praehofer
 */
public class Demo2_ParallelStream {

  private static final String PARALLELISM = "3";
  private static final int MAX_RETURNS = 9;

  static final Random RAND = new Random();

  public static void main(String[] args) {

    System.setProperty("java.util.concurrent.ForkJoinPool.common.parallelism", PARALLELISM);

    long start = System.currentTimeMillis();

    Airline[] airlines = Airline.AIRLINES; 
    LocalDate date = LocalDate.now().plusDays(7); 
    
    List<Flight> flights = 
        Arrays.stream(airlines)
          .parallel()
          .flatMap(airline -> {
            try {
              return Stream.of(airline.findFlight("VIE", "JKF", date));
            } catch (Exception e) {
              return Stream.empty(); 
            }
          })
          .limit(MAX_RETURNS)
          .sorted(Comparator.comparingInt(f -> f.price))
          .collect(Collectors.toList());


     for (Flight flight : flights) {
      System.out.println(flight.toString());
    }

    System.out.format("Execution time: %d ms%n", System.currentTimeMillis() - start);
    
  }

}
