package at.jku.ssw.fp.sect09_2;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;

/**
 * Demos showing the use of {@link Future} for executing asynchronous tasks. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_Future {

  private static final int PARALLELISM = 3;
  static final ExecutorService executor = Executors.newFixedThreadPool(PARALLELISM);
  static final Random RAND = new Random();

  public static void main(String[] args) {

    long start = System.currentTimeMillis();

    Airline[] airlines = Airline.AIRLINES; 
    LocalDate date = LocalDate.now().plusDays(7); 

    List<Future<Flight>> flightFutures = 
        Arrays.stream(airlines)
          .map(airline -> {
            return executor.submit(() -> {
               return airline.findFlight("VIE", "JKF", date);
            });
          })
          .collect(Collectors.toList());


    // continue with some other task

    for (Future<Flight> flightFtr : flightFutures) {
      try {
        Flight flight = flightFtr.get(3, TimeUnit.SECONDS);
        System.out.println(flight.toString());
      } catch (Exception e) { 
        e.printStackTrace();
      } 
    }


    System.out.format("Execution time: %d", System.currentTimeMillis() - start);

    executor.shutdown();
  }

}
