package at.jku.ssw.fp.sect09_4;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;

/**
 * Demo showing the use of asynchronous method {@code findFlightAsync} from {@link Airline}.  
 * 
 * @author Herbert Praehofer
 *
 */
public class Demo_FindFlightAsync {

  static final Random RAND = new Random();

  public static void main(String[] args) throws InterruptedException {
    Airline[] airlines = Airline.AIRLINES; 
    LocalDate date = LocalDate.now().plusDays(7); 

    List<CompletableFuture<Flight>> flightFutures = 
        Arrays.stream(airlines)
          .map(airline -> airline.findFlightAsync("VIE", "JKF", date))
          .collect(Collectors.toList());

    flightFutures.forEach(complFtr -> complFtr.handle((f, ex) -> {
        if (f != null ) {
          System.out.println(f.toString());
          return true;
        } else {
          System.out.println("Exception in " + ex.toString());
          return false;
        }
      })
    );

    Thread.sleep(7000);
    
  }

}
