package at.jku.ssw.fp.sect09_5;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousFileChannel;
import java.nio.channels.CompletionHandler;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.NoSuchElementException;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.stream.Stream;

/**
 * Demo showing the use of {@link CompletableFuture} as promise. 
 * 
 * @author Herbert Praehofer
 */
public class Demo_AsyncFileRead {

  private static final Charset UTF8 = Charset.forName("UTF-8");

  public static void main(String[] args) throws InterruptedException, ExecutionException {
    CompletableFuture<String> rFtr = readFileAsync("files/text.txt");

    String r = rFtr.get();
    System.out.println("content  = \n" + r);

    Thread.sleep(2000);
  }

  public static CompletableFuture<String> readFileAsync(String fn){
    CompletableFuture<String> readComplFtr = 
        new CompletableFuture<String>(); 
    try (AsynchronousFileChannel fileChannel =
             AsynchronousFileChannel.open(Paths.get(fn), 
                                      StandardOpenOption.READ)) {
      ByteBuffer buffer = ByteBuffer.allocate(1024);
      fileChannel.read(buffer, 0, null, 
        new CompletionHandler<Integer, Object>() {
          @Override
          public void completed(Integer result, Object attachment){
            buffer.flip();
            String content = UTF8.decode(buffer).toString();  
            readComplFtr.complete(content); 
           }
          @Override
            public void failed(Throwable exc, Object attachment) {
            readComplFtr.completeExceptionally(
              new IOException("Failed read from " + fn)); 
            }
      });
    } catch (Exception e) {
      readComplFtr.completeExceptionally(
          new IOException("Failed read from " + fn)); 
    }
    return readComplFtr; 
  }


  public static CompletableFuture<String> findInFileAsync(String fileName, String text) {
    CompletableFuture<String> fileNameFtr = new CompletableFuture<String>();
    Demo_AsyncFileRead.readFileAsync(fileName).thenAccept(content -> {
      if (content.contains(text)) {
        fileNameFtr.complete(fileName);
      } else {
        fileNameFtr
            .completeExceptionally(new NoSuchElementException(String.format("Not contained %s in %S", text, fileName)));
      }
    });
    return fileNameFtr;
  }

  public static CompletableFuture<String> findInFileAsyncCond(String fileName, String text) {
    CompletableFuture<String> fileNameFtr = new CompletableFuture<String>();
    Demo_AsyncFileRead.readFileAsync(fileName).thenAccept(content -> {
      if (content.contains(text)) {
        fileNameFtr.complete(fileName);
      } else {
        fileNameFtr
            .completeExceptionally(new NoSuchElementException(String.format("Not contained %s in %S", text, fileName)));
      }
    });
    return fileNameFtr;
  }

  public static String findInFile(String fileName, String text) {
    try (Stream<String> lines = Files.lines(Paths.get(fileName))) {
      if (lines.anyMatch(line -> line.contains(text))) {
        return fileName;
      } else {
        return null;
      }
    } catch (IOException e) {
      return null;
    }
  }

}
