package at.jku.ssw.fp.sect09_6;

import java.time.LocalDate;
import java.util.Random;
import java.util.concurrent.CompletableFuture;

import at.jku.ssw.fp.airline.Agency;
import at.jku.ssw.fp.airline.Airline;

/**
 * Demos showing the composition of {@link CompletableFuture}s. 
 * 
 * @author Herbert Praehofer
 */
public class Demo_Composition {

  static final Random RAND = new Random();

  public static void main(String[] args) throws InterruptedException {

    Airline lufthansa = Airline.AIRLINES[0]; 
    Agency ruefa = Agency.AGENCIES[0]; 
    Agency tui = Agency.AGENCIES[1]; 
    LocalDate date = LocalDate.now().plusDays(7); 

    lufthansa.findFlightAsync("VIE", "JKF", date)
      .thenApply(flight -> 
        ruefa.getOfferAsync(flight)
           .thenCombine(tui.getOfferAsync(flight), 
                 (offer1, offer2) -> {
                  if (offer1.price <= offer2.price) return offer1;
                  else return offer2; 
                })
          .exceptionally(e -> lufthansa.getOffer(flight))
          .thenAccept(offer -> {
            System.out.format("Best offer for flight %s is %d%n", 
                          flight, offer.price);
          })
          .exceptionally(e -> {
              System.out.println("No offer for flight " + flight); 
              return null; 
          }))
      .exceptionally(e -> {
        System.out.println("No flight from airline  " + lufthansa); 
        return null; 
      });


    Thread.sleep(3000);
    
  }

}
