package at.jku.ssw.fp.sect10_1;

import at.jku.ssw.fp.sect07_1.Article;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;

/**
 * Demo showing an observable of articles. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_ArticleObservable {

  public static void main(String[] args) {
    
    Observable<Article> articleObservable = 
        Observable.create(emitter -> {
          try {
            emitter.onNext(new Article("A", 1500));
            emitter.onNext(new Article("B", 700));
            emitter.onNext(new Article("C", 1200));
            emitter.onNext(new Article("D", 500));
            emitter.onNext(new Article("E", 600));
            emitter.onComplete();
          } catch (Exception e) {
            emitter.onError(e); 
          }
        });
    
    articleObservable.subscribe(new Observer<Article>() {

      @Override
      public void onSubscribe(Disposable d) {
        System.out.println("SUBSCRIBED");
      }

      @Override
      public void onNext(Article a) {
        System.out.println(a.toString());
      }

      @Override
      public void onError(Throwable e) {
        System.out.println("ERROR: " + e.toString());
      }

      @Override
      public void onComplete() {
        System.out.println("COMPLETED"); 
      }
    });

  }

}
