package at.jku.ssw.fp.sect10_1;

import java.util.Comparator;

import at.jku.ssw.fp.sect07_1.Article;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;

/**
 * Demos showing processing names of articles. 
 * 
 * @author Herbert Praehofer
 *
 */
public class Demo2_ArticleNamesObservable {

  public static void main(String[] args) {
    
    Observable<Article> articleObservable = 
        Observable.create(emitter -> {
          try {
            emitter.onNext(new Article("A", 1500));
            emitter.onNext(new Article("B", 700));
            emitter.onNext(new Article("C", 1200));
            emitter.onNext(new Article("D", 500));
            emitter.onNext(new Article("E", 600));
            emitter.onComplete();
          } catch (Exception e) {
            emitter.onError(e); 
          }
        });
    
    Observable<String> articleNamesObservable = 
        articleObservable
          .filter(a -> a.price < 1000.0)
          .sorted(Comparator.comparingDouble(article -> article.price))
          .map(a -> a.name);

    articleNamesObservable.subscribe(new Observer<String>() {

      @Override
      public void onSubscribe(Disposable d) {
        System.out.println("SUBSCRIBED");
      }

      @Override
      public void onNext(String s) {
        System.out.println(s.toString());
      }

      @Override
      public void onError(Throwable e) {
        System.out.println("ERROR: " + e.toString());
      }

      @Override
      public void onComplete() {
        System.out.println("COMPLETED"); 
      }
    });

  }

}
