package at.jku.ssw.fp.sect10_1;

import java.time.LocalDate;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import at.jku.ssw.fp.sect07_1.Article;
import io.reactivex.Observable;

/** 
 * Demos showing various forms for creating observables. 
 * 
 * @author Herbert Praehofer
 */
public class Demo5_CreateObservable {

  @SuppressWarnings("unused")
  public static void main(String[] args) {
    
    Observable<Article> articleObservable = 
        Observable.just(
            new Article("A", 1500),  
            new Article("B", 700),
            new Article("C", 1200), 
            new Article("D", 500), 
            new Article("E", 600)
          );

    List<Article> articleList = List.of(
        new Article("A", 1500), 
        new Article("B", 700),
        new Article("C", 1200), 
        new Article("D", 500), 
        new Article("E", 600)
        );  
    Observable<Article> articleObservable2 = 
        Observable.fromIterable(articleList);
    
    int x = 5, y = 0; 
    Observable<Integer> divObservable = 
        Observable.fromCallable(() -> {
          return x / y; 
        });
    
    divObservable.subscribe(        
        r -> System.out.println(r), 
        e -> System.out.println("Error event: " + e)
      );
    
    Observable<Integer> intObservable = Observable.range(1, 100);
    intObservable.subscribe(        
        r -> System.out.println(r), 
        e -> System.out.println("Error event: " + e)
      );
    
    LocalDate nextWeek =  LocalDate.now().plusDays(7);
    Airline lufthansa = Airline.AIRLINES[0]; 
    Observable<Flight> flightObservable = 
        Observable.fromFuture(
            CompletableFuture.supplyAsync(() -> {
              return lufthansa.findFlight("VIE", "JFK", nextWeek); 
            }), 
            3, TimeUnit.SECONDS); 
    
    Observable<Long> secsObs = 
        Observable.interval(1, TimeUnit.SECONDS); 

    Observable.generate(() -> new StringBuilder(), (b, emitter) -> {
      if (b.length() > 10) emitter.onComplete();
      else {
        b.append("A"); 
        emitter.onNext(b.toString());
      }
    })
    .subscribe(
        System.out::println, 
        System.out::println, 
        () -> System.out.println("Generate COMPLETED")
     ); 
    
  }

}
