package at.jku.ssw.fp.sect10_1;

import static at.jku.ssw.fp.sect10.Constants.executor;

import java.time.LocalDate;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Timed;

/**
 * Demos showing various ways to subscribe to observables.
 *
 * @author Herbert Praehofer
 */
public class Demo6_Subscribe {

  public static void main(String[] args) {

    LocalDate nextWeek =  LocalDate.now().plusDays(7);
    Airline[] airlines = Airline.AIRLINES;

    Observable<Flight> flightObservable =
        Observable.create(emitter -> {
          AtomicInteger nFtrs =
              new AtomicInteger(airlines.length);
          for (Airline airline : airlines) {
            CompletableFuture.supplyAsync(() -> {
                  return airline.findFlight("VIE", "JKF", nextWeek);
                }, executor)
                .whenComplete((flight, excpt) -> {
                  synchronized (emitter) {
                    if (flight != null)
                      emitter.onNext(flight);
                    if (nFtrs.decrementAndGet() == 0)
                        emitter.onComplete();
                    }

                });
          }
       });

    Observable<Timed<Flight>> timedFlightObservable =
        flightObservable.timestamp();

    timedFlightObservable.subscribe(
        flight -> System.out.println(flight),
        ex -> System.out.println("Exception in " + ex.toString()),
        () -> System.out.println("COMPLETED"),
        disposable -> { }
      );

    flightObservable.subscribe(
        flight -> System.out.println(flight),
        ex -> System.out.println("Exception in " + ex.toString()),
        () -> System.out.println("COMPLETED"),
        disposable -> { }
      );

    flightObservable.subscribe(
        flight -> System.out.println(flight),
        ex -> System.out.println("Exception in " + ex.toString())
      );

    flightObservable
      .forEach(flight -> System.out.println(flight));

    flightObservable.subscribe(new Observer<Flight>() {
      private Disposable disposable;

      @Override
      public void onSubscribe(Disposable disposable) {
        this.disposable = disposable;
      }

      @Override
      public void onNext(Flight flight) {
        if (fitsCriteria(flight)) {
          System.out.println("FOUND: " + flight.toString());
          disposable.dispose();
        }
      }

      @Override
      public void onError(Throwable excpt) {
        System.out.println("Exception in " + excpt.toString());
      }

      @Override
      public void onComplete() {
        System.out.println("COMPLETED");
      }

      private boolean fitsCriteria(Flight flight) {
        return flight.price < 300;
      }

    });

  }

}
