package at.jku.ssw.fp.sect10_2;

import static at.jku.ssw.fp.sect10.Constants.executor;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.stream.Collectors;

import io.reactivex.Single;
import io.reactivex.SingleObserver;
import io.reactivex.disposables.Disposable;

/**
 * Demos showing the use of {@link Single}.
 *
 * @author Herbert Praehofer
 */
public class Demo1_Single {

  public static void main(String[] args) {

    URL url;
    try {
      url = new URL(ADDRESS);
      Single<String> pageSingle = Single.create(emitter -> {
        executor.submit(() -> {
          try {
            String content = getWebPage(url);
            emitter.onSuccess(content);
          } catch (Exception ex) {
            emitter.onError(ex);
          }
        });
      });

      pageSingle.subscribe(new SingleObserver<String>() {
        @Override
        public void onSubscribe(Disposable d) {
        }

        @Override
        public void onSuccess(String page) {
          System.out.println(page);
        }

        @Override
        public void onError(Throwable e) {
          e.printStackTrace();
        }
      });

      Thread.sleep(3000);

      executor.shutdown();

    } catch (Exception e1) {
      e1.printStackTrace();
    }

  }

  private static final String ADDRESS = "http://www.ssw.uni-linz.ac.at/General/Staff/HP/index.html";

  static String getWebPage(URL url) throws IOException {
    try (BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()))) {
      return reader.lines().collect(Collectors.joining("\n"));
    }
  }

}
