package at.jku.ssw.fp.sect10_2;

import static at.jku.ssw.fp.sect10.Constants.executor;

import java.util.Random;

import io.reactivex.Completable;
import io.reactivex.CompletableObserver;
import io.reactivex.disposables.Disposable;

/**
 * Demos showing the use of {@link Completable}. 
 * 
 * @author Herbert Praehofer
 */
public class Demo2_Completable {
  
	static final Random RAND = new Random(); 

	static void updateDB(String sqlStmt) throws Exception {
		// Add db update here and possible throw exception 
		if (RAND.nextBoolean()) throw new Exception("update failed"); 
		try {
			Thread.sleep(1200);
		} catch (InterruptedException e) {	}
		return; 
	}
	
	static Completable updateDBCompletable(String sqlStmt) {
		return Completable.create(emitter -> {
			executor.submit(() -> {
				try {
					updateDB(sqlStmt); 
					emitter.onComplete(); 
				} catch (Exception e) {
					emitter.onError(e);
				}
			});
		}); 
	}
	
	public static void main(String[] args) throws Exception {

	  String sqlStmt = "UPDATE"; 
	  
	  Completable dbCpltble = 
	      Completable.create(emitter -> {
	        executor.submit(() -> {
	          try {
	            updateDB(sqlStmt); 
	            emitter.onComplete(); 
	          } catch (Exception e) {
	            emitter.onError(e);
	          }
	        });
	      });
	  
	  dbCpltble.subscribe(
	      new CompletableObserver() {
	        @Override
	        public void onSubscribe(Disposable d) {  }
	        @Override
	        public void onComplete() {
	          System.out.println("UPDATE SUCCESSFUL"); 
	        }
	        @Override
	        public void onError(Throwable e) {
	          System.out.println("UPDATE FAILED"); 
	        }
	      }); 


		Thread.sleep(2000);

    executor.shutdown();

	}

}
