package at.jku.ssw.fp.sect10_2;

import static at.jku.ssw.fp.sect10.Constants.executor;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Maybe;
import io.reactivex.MaybeObserver;
import io.reactivex.disposables.Disposable;

/**
 * Demos showing the use of {@link Maybe}. 
 * 
 * @author Herbert Praehofer
 */
public class Demo3_Maybe {
  
  static Airline[] airlines = Airline.AIRLINES;

	static Optional<Flight> findFlight(String from, String to, LocalDate date) throws Exception {
		List<CompletableFuture<Flight>> flightFutures = 
				Arrays.stream(airlines)
					.map(airline -> {
							return CompletableFuture.supplyAsync(() -> {
								return airline.findFlight(from, to, date);
						}, 
						executor);
					})
					.collect(Collectors.toList());
		try {
			return Optional.ofNullable(
					(Flight)CompletableFuture.anyOf(flightFutures.toArray(new CompletableFuture[0]))
						.get(5, TimeUnit.SECONDS));
		} catch (Exception e) {
			return Optional.empty(); 
		}
	}

	public static void main(String[] args) throws Exception {
		LocalDate nextWeek =  LocalDate.now().plusDays(7);
		
		Maybe<Flight> flightMaybe = 
	    Maybe.create(emitter -> {
	      executor.submit(() -> {
	        try {
	           Optional<Flight> optFlight = 
	             findFlight("VIE", "JFK", nextWeek); 
  	        if (optFlight.isPresent()) {
  	          emitter.onSuccess(optFlight.get());
  	        } else {
  	          emitter.onComplete();
  	        }
  	      } catch (Exception e) {
  	        emitter.onError(e);
  	      }
	      }); 
	    });

 
		flightMaybe.subscribe(
		    new MaybeObserver<Flight>() {
		      @Override
		      public void onSubscribe(Disposable d) {}
		      @Override
		      public void onSuccess(Flight flight) {
		        System.out.println("Flight found is " + flight);
		      }
		      @Override
		      public void onComplete() {
		        System.out.println("No flight found!");      
		      }
		      @Override
		      public void onError(Throwable e) {
		        System.out.println("Find flight failed with " + e);
		      }
		    });

		
		Thread.sleep(2000);
		
    executor.shutdown();

	}
			
}
