package at.jku.ssw.fp.sect10_3;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Comparator;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Observable; 

/**
 * Demos showing a cold observable. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_ColdObservable {

    private static ExecutorService executor = Executors.newWorkStealingPool();
  
  	public static void main(String[] args) throws InterruptedException {
	   
		LocalDate nextWeek =  LocalDate.now().plusDays(7); 

		long start = System.currentTimeMillis();
		
		Observable<Flight> flightObservable = 
				Observable.create(emitter -> {
					AtomicInteger nFtrs = new AtomicInteger(Airline.AIRLINES.length);
					Arrays.stream(Airline.AIRLINES).forEach(airline -> {
							CompletableFuture.supplyAsync(() -> {
								System.out.println("findFlight " + airline.name + ": " + Thread.currentThread().getName()); 
								return airline.findFlight("VIE", "JKF", nextWeek);
							}, executor)
							.whenComplete((flight, excpt) -> {
								synchronized (emitter) {
									if (flight != null) 
										emitter.onNext(flight);
									if (nFtrs.decrementAndGet() == 0) 
										emitter.onComplete();
								}
							});
						});
					}); 
		
		flightObservable.subscribe(
				flight -> System.out.println("ONNEXT: " + flight.toString()), 
				excpt -> System.out.println("ONERROR: " + "Exception in " + excpt.toString()),
				() -> System.out.println("COMPLETED")
			);

		flightObservable
			.sorted(Comparator.comparingInt(flight -> flight.price))
			.toList()
			.subscribe(System.out::println);
		

		executor.shutdown();
		executor.awaitTermination(20, TimeUnit.SECONDS);	

		System.out.format("Execution time: %d", System.currentTimeMillis() - start);

	}

}
