package at.jku.ssw.fp.sect10_3;

import static at.jku.ssw.fp.sect10.Constants.executor;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import io.reactivex.observables.ConnectableObservable;

/**
 * Demo showing how to use {@link ConnectableObservable#autoConnect()}.
 *
 * @author Herbert Praehofer
 */
public class Demo3_Autoconnect {

	public static void main(String[] args) throws InterruptedException {

		LocalDate nextWeek =  LocalDate.now().plusDays(7);

		long start = System.currentTimeMillis();

		Observable<Flight> flightObservable =
				Observable.create(emitter -> {
					AtomicInteger nFtrs = new AtomicInteger(Airline.AIRLINES.length);
					Arrays.stream(Airline.AIRLINES).forEach(airline -> {
							CompletableFuture.supplyAsync(() -> {
//								System.out.println("findFlight " + airline.name + ": " + Thread.currentThread().getName());
								return airline.findFlight("VIE", "JKF", nextWeek);
							}, executor)
							.whenComplete((flight, excpt) -> {
								synchronized (emitter) {
									if (flight != null)
										emitter.onNext(flight);
									if (nFtrs.decrementAndGet() == 0)
										emitter.onComplete();
								}
							});
						});
					});

		Observable<Flight> autoConnectingObservable =
	       flightObservable.publish().autoConnect(2);

		System.out.println("AUTOCONNECT");

		autoConnectingObservable.subscribe(new Observer<Flight>() {

			@Override
			public void onSubscribe(Disposable d) {
				System.out.println("ONSUBSCRIBE 1");
			}

			@Override
			public void onNext(Flight flight) {
				System.out.println("ONNEXT 1: " + flight.toString());
			}

			@Override
			public void onError(Throwable excpt) {
				System.out.println("ONERROR 1: " + "Exception in " + excpt.toString());
			}

			@Override
			public void onComplete() {
				System.out.println("COMPLETED 1");
			}

		});
		System.out.println("SUBSCRIBED 1");

		System.out.println("-----WAITING 1000 -------------");
		Thread.sleep(1000);
		System.out.println("-----WAITED 1000 -------------");


		autoConnectingObservable.subscribe(new Observer<Flight>() {

			@Override
			public void onSubscribe(Disposable d) {
				System.out.println("ONSUBSCRIBE 2");
			}

			@Override
			public void onNext(Flight flight) {
				System.out.println("ONNEXT 2: " + flight.toString());
			}

			@Override
			public void onError(Throwable excpt) {
				System.out.println("ONERROR 2: " + "Exception in " + excpt.toString());
			}

			@Override
			public void onComplete() {
				System.out.println("COMPLETED 2");
			}

		});
		System.out.println("SUBSCRIBED 2");


		System.out.println("-----WAITING 5000 -------------");
		Thread.sleep(5000);
		System.out.println("-----WAITED 5000 -------------");


		autoConnectingObservable.subscribe(new Observer<Flight>() {

			@Override
			public void onSubscribe(Disposable d) {
				System.out.println("ONSUBSCRIBE 3");
			}

			@Override
			public void onNext(Flight flight) {
				System.out.println("ONNEXT 3: " + flight.toString());
			}

			@Override
			public void onError(Throwable excpt) {
				System.out.println("ONERROR 3: " + "Exception in " + excpt.toString());
			}

			@Override
			public void onComplete() {
				System.out.println("COMPLETED 3");
			}

		});
		System.out.println("SUBSCRIBED 3");

		executor.shutdown();
		executor.awaitTermination(20, TimeUnit.SECONDS);

		System.out.format("Execution time: %d", System.currentTimeMillis() - start);

	}

}
