package at.jku.ssw.fp.sect10_3;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import io.reactivex.Observable;

/**
 * Demo showing an observable of weather data. 
 * 
 * @author Herbert Praehofer
 */

public class Demo6_WeaterObservable {
	
	private static String addresss = "https://samples.openweathermap.org/data/2.5/weather?q=London,uk&appid=b6907d289e10d714a6e88b30761fae22";

    public static void main(String[] args) throws Exception {
		
		URL url = new URL(addresss); 
		
		Observable<WeatherData> weatherDataObservable = 
		    Observable
		      .interval(10, TimeUnit.SECONDS)
		      .map(i -> getWeatherData(url))
		      .map(json -> WeatherData.fromJSON(json))
		      .share(); 
		
		Observable<Double> tFahrenht = 
		    weatherDataObservable
		        .map(wData -> wData.temp)
		        .map(k -> k * 9/5 - 459.67);  
					
		Observable<Double> tCelsius = 
		    weatherDataObservable
		        .map(wData -> wData.temp)
		        .map(k -> k - 273.15);  
 
		
		tFahrenht.subscribe(f -> System.out.println("Fahrenheit: "+f));
		tCelsius.subscribe(c -> System.out.println("Celsius: " + c));
		
		Thread.sleep(400_000);
		
	}

	private static String getWeatherData(URL url) throws IOException {
		try (BufferedReader r = new BufferedReader(new InputStreamReader(url.openStream()))) {
			return r.lines().collect(Collectors.joining("\n")); 
		}
	}

}
