package at.jku.ssw.fp.sect10_4;

import java.time.LocalDate;
import java.util.List;
import java.util.concurrent.CompletableFuture;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers; 

/**
 * Demo shows flatMap operations. 
 * 
 * @author Herbert Praehofer
 *
 */
public class Demo1_FlatMap {
	
	static Observable<Flight> getFlightObservable(Airline airline, String from, String to, LocalDate date) {
		return Observable.fromFuture(
				CompletableFuture.supplyAsync(() -> {
					return airline.findFlight(from, to, date); 
				}))
				.subscribeOn(Schedulers.io());
	}
	
	static Observable<Flight> getFlightsObservable(Airline airline, String from, String to, LocalDate date) {
		Observable<List<Flight>> flightsObservable = Observable.fromFuture(
				CompletableFuture.supplyAsync(() -> {
					return airline.findFlights(from, to, date); 
				}))
				.subscribeOn(Schedulers.io());
		return flightsObservable.flatMapIterable(list -> list); 
	}
	
	static Observable<Flight> getFlightObservableSync(Airline airline, String from, String to, LocalDate date) {
		return Observable.just(airline.findFlight(from, to, date)) 
					.subscribeOn(Schedulers.io());
	}
	
	public static void main(String[] args) throws InterruptedException {
	  
		LocalDate nextWeek =  LocalDate.now().plusDays(7); 

		Observable<Flight> flightsObservable = 
			Observable.fromArray(Airline.AIRLINES)
				.flatMap(airline -> getFlightsObservable(airline, "VIE", "JKF", nextWeek)); 
		
		flightsObservable.subscribe(new Observer<Flight>() {

			@Override
			public void onSubscribe(Disposable d) {
				System.out.println("ONSUBSCRIBE");
			}

			@Override
			public void onNext(Flight flight) {
				System.out.println("onNext: " + Thread.currentThread().getName()); 
				System.out.println("ONNEXT: " + flight.toString());
			}

			@Override
			public void onError(Throwable excpt) {
				System.out.println("onError: " + Thread.currentThread().getName()); 
				System.out.println("ONERROR: " + "Exception in " + excpt.toString());
			}

			@Override
			public void onComplete() {
				System.out.println("onComplete: " + Thread.currentThread().getName()); 
				System.out.println("COMPLETED");
			}
			
		});

		Thread.sleep(20_000);
	}
			
}
