package at.jku.ssw.fp.sect10_4;

import java.util.Collection;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.Single;
import io.reactivex.observables.GroupedObservable; 

/**
 * Demo showing various collect operations. 
 * 
 * @author Herbert Praehofer
 */
public class Demo4_CollectOperations {
	
	private static final int PARALLELISM = 10;
	private static final ExecutorService executor = Executors.newFixedThreadPool(PARALLELISM); 
	private static final Random RAND = new Random(); 

	public static void main(String[] args) throws InterruptedException {
	  
	  // toMultiMap
	  
	  Observable<String> nameObservable = Observable.just("Hans", "Berta", "Hugo", "Johann", "Josef", "Anton", "Susi", "Gust", "Gabi", "Xaver"); 
	  
	  Single<Map<Character, Collection<String>>> wordGroupsSingle = 
	      nameObservable
	        .filter (name -> name.length() > 0)
	        .toMultimap(word -> Character.toLowerCase(word.charAt(0)));


	  System.out.println(wordGroupsSingle.blockingGet()); 

	  // groupBy 
	  
		Observable<Integer> oneTwoThreeObs = 
				Observable
					.interval(1,  TimeUnit.SECONDS)
					.map(i -> RAND.nextInt(3) +1)
					.doOnNext(System.out::println); 

		Observable<GroupedObservable<Integer, Integer>> groupedObs =
		    oneTwoThreeObs.groupBy(x -> x);

		
		groupedObs.subscribe(
		    (GroupedObservable<Integer, Integer> groupObs)  -> {
		      groupObs
		        .take(3)
		        .toList()
		        .subscribe(System.out::println); 
		      });


		Thread.sleep(11000);
		executor.shutdown();
		executor.awaitTermination(20, TimeUnit.SECONDS);	
		Thread.sleep(40000);
	}
	
}
