package at.jku.ssw.fp.sect10_4;

import static java.util.concurrent.TimeUnit.MILLISECONDS;

import java.util.concurrent.TimeUnit;

import at.jku.ssw.fp.util.LongRef;
import io.reactivex.Observable;
import io.reactivex.schedulers.Timed;

/**
 * Demo showing the use of {@link Observable#timestamp()}.
 *
 * @author Herbert Praehofer
 */
public class Demo6_Timestamp {

	public static void main(String[] args) throws InterruptedException {

		Observable<Integer> queueLengthObservable =
				Observable.interval(0, 1, TimeUnit.SECONDS).zipWith(Observable.just(0, 1, 0, 1, 0, 1, 0), (i, v) -> v);

		LongRef start = LongRef.of(System.currentTimeMillis());
		Observable<Double> meanQueueLengthObservable =
		    queueLengthObservable
		      .doOnSubscribe(x -> {
		        start.value = System.currentTimeMillis();
		      })
		      .timestamp()
		      .map(tV -> new Timed<Integer>(tV.value(),
		                         tV.time() - start.value, MILLISECONDS))
		      .doOnNext(System.out::println)
		      .buffer(2, 1)
		      .map(tVBuf -> {
		        long time0 = tVBuf.get(0).time(MILLISECONDS);
		        if (tVBuf.size() > 1) {
		          long time1 = tVBuf.get(1).time(MILLISECONDS);
		          long timeSpan = time1 - time0;
		          double product = timeSpan * tVBuf.get(0).value();
		          return new Timed<Double>(product, time1, MILLISECONDS);
		        } else {
		          return new Timed<Double>(0.0, time0, MILLISECONDS);
		        }
		      })
		      .scan((tSum, tV) -> {
		        return new Timed<Double>(tSum.value() + tV.value(),
		                                 tV.time(), MILLISECONDS);
		      })
		      .map(tSum -> tSum.value() / tSum.time());

		meanQueueLengthObservable
			.subscribe(System.out::println);


		Thread.sleep(20000);

	}

}
