package at.jku.ssw.fp.sect10_5;

import static at.jku.ssw.fp.sect10.Constants.executor;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;

/**
 * Demo program showing the serialization of events by method {@link Observable#serialize}}.
 *
 * @author Herbert Praehofer
 */
public class Demo1_Serialize {

	public static void main(String[] args) {

		LocalDate nextWeek =  LocalDate.now().plusDays(7);

		Airline[] airlines = Airline.AIRLINES;

		Observable<Flight> flightObservable =
		    Observable.<Flight>create(emitter -> {
		      AtomicInteger nFtrs =
		          new AtomicInteger(airlines.length);
		      Arrays.stream(airlines).forEach(airline -> {
		        CompletableFuture.<Flight>supplyAsync(() ->
		            airline.findFlight("VIE", "JKF", nextWeek), executor)
		          .whenComplete((flight, excpt) -> {
		            if (flight != null) emitter.onNext(flight);
		            if (nFtrs.decrementAndGet() == 0) emitter.onComplete();
		          });
		        });
		      }).serialize();


		flightObservable.subscribe(new Observer<Flight>() {

			@Override
			public void onSubscribe(Disposable d) {
				System.out.println("ONSUBSCRIBE");
			}

			@Override
			public void onNext(Flight flight) {
				System.out.println("onNext: " + Thread.currentThread().getName());
				System.out.println("ONNEXT: " + flight.toString());
			}

			@Override
			public void onError(Throwable excpt) {
				System.out.println("onError: " + Thread.currentThread().getName());
				System.out.println("ONERROR: " + "Exception in " + excpt.toString());
			}

			@Override
			public void onComplete() {
				System.out.println("onComplete: " + Thread.currentThread().getName());
				System.out.println("COMPLETED");
			}

		});

	}

}
