package at.jku.ssw.fp.sect10_5;

import static at.jku.ssw.fp.airline.Airline.AIRLINES;

import java.time.LocalDate;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

/**
 * Demo program showing the how to use {@link Observable#flatMap} for
 * asynchronous event processing.
 *
 * @author Herbert Praehofer
 */
public class Demo3_AsyncWithFlatMap {

	public static void main(String[] args) throws InterruptedException {
		LocalDate nextWeek =  LocalDate.now().plusDays(7);
		long start = System.currentTimeMillis();

    Airline[] airlines = AIRLINES;

    Observable<Flight> flightObservable =
		    Observable
		      .fromArray(airlines)
		      .flatMap(airline ->  Observable.fromCallable(
		            () -> airline.findFlight("VIE", "JKF", nextWeek))
		      .subscribeOn(Schedulers.io()) );

		flightObservable.subscribe(new Observer<Flight>() {

					@Override
					public void onSubscribe(Disposable d) {
						System.out.println("ONSUBSCRIBE");
					}

					@Override
					public void onNext(Flight flight) {
						System.out.println("onNext: " + Thread.currentThread().getName());
						System.out.format("onNext time: %d%n", System.currentTimeMillis() - start);
						System.out.println("ONNEXT: " + flight.toString());
					}

					@Override
					public void onError(Throwable excpt) {
						System.out.println("onError: " + Thread.currentThread().getName());
						System.out.println("ONERROR: " + "Exception in " + excpt.toString());
					}

					@Override
					public void onComplete() {
						System.out.println("onComplete: " + Thread.currentThread().getName());
						System.out.format("onComplete time: %d%n", System.currentTimeMillis() - start);
						System.out.println("COMPLETED");
					}

				});
		System.out.println(Thread.currentThread().getName() + ": " + "Subscribed!");

		Thread.sleep(4_000);

	}

}
