package at.jku.ssw.fp.sect10_5;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import at.jku.ssw.fp.sect10_3.WeatherData;
import io.reactivex.Observable;

/**
 * Demo program showing the how to use {@link Observable#observeOn} for
 * asynchronous event handling in consumers.
 *
 * @author Herbert Praehofer
 */

public class Demo4_ObserveOn {

	private static String addresss = "https://api.openweathermap.org/data/2.5/weather?appid=f0d81b7ec9a918a6c4e01fcba9fe537a&lat=48&lon=14";

	private static int N = 60;

	private JFrame frame;
	private javax.swing.JList<String> temps;

	private Demo4_ObserveOn(Observable<WeatherData> weatherDataObservable) {

		frame = new JFrame("Temperature");
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		DefaultListModel<String> model = new DefaultListModel<String>();
		temps = new JList<String>(model);
		frame.getContentPane().add(new JScrollPane(temps));

		weatherDataObservable
			.map(wData -> wData.temp)
			.map(f -> (f - 32.0) * 5.0 / 9.0)
			.timestamp()
			.observeOn(SwingScheduler.getInstance())
			.subscribe(timeTemp -> {
				System.out.println("THREAD: " + Thread.currentThread().getName());
				System.out.println("DATA:   " + timeTemp);
				if (model.getSize() == N) {
					model.remove(N - 1);
				}
				LocalDateTime time =
					    LocalDateTime.ofInstant(Instant.ofEpochMilli(timeTemp.time()), ZoneId.systemDefault());
				model.add(0, String.format("%s =  %4.1f", time.format(DateTimeFormatter.ISO_LOCAL_TIME),
						timeTemp.value()));
			});

		frame.pack();
		frame.setVisible(true);
	}

	public static void main(String[] args) {
	  try {
  		URL url = new URL(addresss);

  		Observable<WeatherData> weatherDataObservable =
  				Observable
  					.interval(0, 10, TimeUnit.SECONDS)
  					.map(i -> getWeatherData(url))
  					.map(json -> WeatherData.fromJSON(json))
  					.publish()
  					.refCount();

  		@SuppressWarnings("unused")
      Demo4_ObserveOn app = new Demo4_ObserveOn(weatherDataObservable);
	  } catch (Exception e) {
	    e.printStackTrace();
	  }

	}

	private static String getWeatherData(URL url) throws IOException {
		try (BufferedReader r = new BufferedReader(new InputStreamReader(url.openStream()))) {
			return r.lines().collect(Collectors.joining("\n"));
		}
	}

}
