package at.jku.ssw.fp.sect10_6;

import java.util.List;

import io.reactivex.Observable; 

/**
 * Demo showing error handling. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_OnError {
	
	public static void main(String[] args) throws InterruptedException {

		List<Integer> numbers = List.of(3, 10, 5, 0, 2, 4, 6); 
		
		// --- try-catch im Emitter-Code  ---
		
		System.out.println("try-catch im Emitter-Code:");

		Observable<Integer> quotObservable1 = 
		    Observable.create(emitter -> {
		      for (int x : numbers) {
		        try {
		           emitter.onNext(100 / x); 
		        } catch (Exception e) {
		           emitter.onError(e); 
		        }
		      }        
		    });
		
		quotObservable1.subscribe(
					div -> System.out.println("Result = " + div),
					excpt -> System.out.println("Error: " + excpt), 
					() -> System.out.println("Completed!")
				); 
	
		// --- Exception in map  ---
		
		System.out.println("\nException in map:");

		Observable<Integer> quotObservable2 = 
		    Observable.fromIterable(numbers)
          .map(x -> 100 / x);
 
		quotObservable2.subscribe(
					div -> System.out.println("Result = " + div),
					excpt -> System.out.println("Error: " + excpt), 
					() -> System.out.println("Completed!")
				); 
	
		// --- Exception in flatMap  ---

		System.out.println("\nException in flatMap:");

		Observable<Integer> quotObservable3 = 
		    Observable.fromIterable(numbers)
  		    .flatMap(x -> 
  		      (x != 0) 
  		          ? Observable.just(100 / x) 
  		          : Observable.error(new ArithmeticException("/ zero")));
		
		quotObservable3.subscribe(
					div -> System.out.println("Result = " + div),
					excpt -> System.out.println("Error: " + excpt), 
					() -> System.out.println("Completed!")
				); 
	
		// --- Ignore with flatMap  ---
		
		System.out.println("\nIgnorieren mit flatMap:");

		Observable<Integer> quotObservable4 = 
		    Observable.fromIterable(numbers)
  		    .flatMap(x -> 
  		      (x != 0) ? Observable.just(100 / x) 
  		               : Observable.empty());
		
		quotObservable4.subscribe(
					div -> System.out.println("Result = " + div),
					excpt -> System.out.println("Error: " + excpt), 
					() -> System.out.println("Completed!")
				); 
	
	}
	
}
