package at.jku.ssw.fp.sect10_6;

import static at.jku.ssw.fp.sect10.Constants.TIMEOUT_DELAY;
import static at.jku.ssw.fp.sect10.Constants.TIMEOUT_PROP;

import java.time.LocalDate;
import java.util.concurrent.TimeUnit;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import at.jku.ssw.fp.util.Util;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers; 

/**
 * Demo showing timeouts. 
 * 
 * @author Herbert Praehofer
 */
public class Demo2_Timeouts {

	public static void main(String[] args) throws InterruptedException {
		
		TIMEOUT_PROP = 0.3; 
		TIMEOUT_DELAY = 10_000; 
		
		LocalDate nextWeek =  LocalDate.now().plusDays(7); 

		Observable<Flight> flightObservable = 
				Observable.fromArray(Airline.AIRLINES)
				.flatMap(airline -> {
					return Observable.fromCallable(() -> {
									return airline.findFlight("VIE", "JKF", nextWeek);
								})
							.subscribeOn(Schedulers.io()); 
				});  
		
		Util.log("Start"); 
		flightObservable
			.timeout(5, TimeUnit.SECONDS)
			.subscribe(new Observer<Flight>() {
	
				@Override
				public void onSubscribe(Disposable d) {
					Util.log("onSubscribe"); 
					System.out.println("ONSUBSCRIBE");
				}
	
				@Override
				public void onNext(Flight flight) {
					Util.log("onNext"); 
					System.out.println("ONNEXT: " + flight.toString());
				}
	
				@Override
				public void onError(Throwable excpt) {
					Util.log("onError"); 
					System.out.println("ONERROR: " + "Exception " + excpt);
				}
	
				@Override
				public void onComplete() {
					Util.log("onComplete"); 
					System.out.println("COMPLETED");
				}
				
			});

		System.out.println(Thread.currentThread().getName() + ": " + "Subscribed!"); 
		
		Thread.sleep(10_000);

	}
	
}
