package at.jku.ssw.fp.sect10_6;

import static at.jku.ssw.fp.sect10.Constants.EXCPT_PROP;
import static at.jku.ssw.fp.sect10.Constants.TIMEOUT_PROP;

import java.time.LocalDate;

import at.jku.ssw.fp.airline.Agency;
import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.airline.Flight;
import at.jku.ssw.fp.util.Util;
import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

/**
 * Demo showing error handling with {@link Observable#onErrorResumeNext}.
 *
 * @author Herbert Praehofer
 */
public class Demo5_OnErrorResumeNext {

	public static void main(String[] args) throws InterruptedException {

		TIMEOUT_PROP = 0.0;
		EXCPT_PROP = 0.5;
		Agency agency = Agency.AGENCIES[0];
		LocalDate nextWeek =  LocalDate.now().plusDays(7);
		Airline[] airlines = Airline.AIRLINES;

		Observable<Flight> flightObservable =
		    Observable.fromArray(airlines)
		      .flatMap(airline -> {
		          return Observable.fromCallable(() ->
		               airline.findFlight("VIE", "JKF", nextWeek))
		          .subscribeOn(Schedulers.io())
		          .onErrorResumeNext(Observable.fromCallable(() ->
		               agency.findFlight(airline, "VIE", "JKF", nextWeek)))
		          .onErrorResumeNext(Observable.empty());
		      });


		Util.log("Start");
		flightObservable
			.subscribe(new Observer<Flight>() {

				@Override
				public void onSubscribe(Disposable d) {
					Util.log("onSubscribe");
					System.out.println("ONSUBSCRIBE");
				}

				@Override
				public void onNext(Flight flight) {
					Util.log("onNext");
					System.out.println("ONNEXT: " + flight.toString());
				}

				@Override
				public void onError(Throwable excpt) {
					Util.log("onError");
					System.out.println("ONERROR: " + "Exception " + excpt);
				}

				@Override
				public void onComplete() {
					Util.log("onComplete");
					System.out.println("COMPLETED");
				}

			});

		System.out.println(Thread.currentThread().getName() + ": " + "Subscribed!");

		Thread.sleep(10_000);

	}
}
