package at.jku.ssw.fp.sect10_7;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Scanner;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.stream.Stream;

import io.reactivex.BackpressureStrategy;
import io.reactivex.Flowable;
import io.reactivex.schedulers.Schedulers;

/**
 * Demo program implementing a streaming of a text with backpressure. 
 * 
 * @author Herbert Praehofer
 */
public class Demo3_StreamingSong {
	
	private static final int DELAY = 20;
	private static CountDownLatch teriminateLatch; 
	
	public static void main(String[] args) throws InterruptedException {
		
		try (Scanner scanner = new Scanner(System.in)) {
		  
			System.out.print("Bitte Songdatei eingeben (adele | cohen | sinatra): ");
			String input = scanner.nextLine().trim(); 
			String fileName = "songs\\" + input + ".txt"; 
			if (Files.notExists(Paths.get(fileName))) {
				System.out.println(String.format("File %s existiert nicht ", fileName));
				return; 
			} 
			
			Flowable<Character> charFlowable = 
			    Flowable.<Character>create(emitter -> {
			      try (Stream<String> lines = 
			                Files.lines(Paths.get(fileName))) {
			        lines.map(l -> l + "\n")
			             .flatMapToInt(l -> l.chars())
			             .forEach(c -> emitter.onNext((char)c));
			        emitter.onComplete();
			      } catch (Exception e) {
			        emitter.onError(e);
			      }
			    }, BackpressureStrategy.BUFFER)
			    .subscribeOn(Schedulers.io());
 
			teriminateLatch = new CountDownLatch(1); 
			
			Flowable.interval(DELAY, TimeUnit.MILLISECONDS)
	  			.zipWith(charFlowable, (i, c) -> c)
	  			.observeOn(Schedulers.computation())
	  			.subscribe(
	  			    c -> System.out.print(c),
	  			    e -> {
	  			    	System.out.println("ERROR: " + e); 
	  			    	teriminateLatch.countDown();
	  			    }, 
	  			    () -> {
	  			    	System.out.println("END"); 
	  			    	teriminateLatch.countDown(); 
	  			    }
	  			);
			
			
			teriminateLatch.await(); 
		}
	}
	
}
