package at.jku.ssw.fp.coll.test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.NoSuchElementException;

import org.junit.jupiter.api.Test;

import at.jku.ssw.fp.coll.FList;

/** 
 * Tests of {@link FList}
 * 
 * @author Herbert Praehofer
 */
class FListTest {

    @Test
    void testIsEmpty() {
        FList<String> l = FList.of("A", "B", "C");
        assertFalse(l.isEmpty());
        assertFalse(l.tail().isEmpty());
        assertFalse(l.tail().tail().isEmpty());
        assertTrue(l.tail().tail().tail().isEmpty());
    }

    @Test
    void testSize() {
        FList<String> l = FList.of("A", "B", "C");
        assertEquals(3, l.size());
        assertEquals(2, l.tail().size());
        assertEquals(1, l.tail().tail().size());
        assertEquals(0, l.tail().tail().tail().size());
    }

    @Test
    void testIterator() {
        String[] strgs = { "A", "B", "C" };
        FList<String> l = FList.of("A", "B", "C");
        int i = 0;
        for (String e : l) {
            assertEquals(strgs[i], e);
            i++;
        }
    }

    @Test
    void testAdd() {
        FList<String> l = FList.empty();
        assertEquals(FList.empty(), l);

        l = l.add("C");
        assertEquals(FList.of("C"), l);

        l = l.add("B");
        assertEquals(FList.of("B", "C"), l);

        l = l.add("A");
        assertEquals(FList.of("A", "B", "C"), l);

        l = l.tail();
        assertEquals(FList.of("B", "C"), l);

        l = l.tail();
        assertEquals(FList.of("C"), l);

        l = l.tail();
        assertEquals(FList.empty(), l);

    }

    @Test
    void testToString() {
        FList<String> l = FList.empty();
        assertEquals("[]", l.toString());

        l = l.add("C");
        assertEquals("[C]", l.toString());

        l = l.add("B");
        assertEquals("[B, C]", l.toString());

        l = l.add("A");
        assertEquals("[A, B, C]", l.toString());

    }

    @Test
    void testHead() {
        FList<String> l = FList.of("A", "B", "C");
        assertEquals("A", l.head());
        assertEquals("B", l.tail().head());
        assertEquals("C", l.tail().tail().head());
    }

    @Test
    void testExcpts() {
        FList<String> l = FList.of("A", "B", "C");
        assertThrows(NoSuchElementException.class, () -> {
            l.tail().tail().tail().head();
        });
        assertThrows(NoSuchElementException.class, () -> {
            l.tail().tail().tail().tail();
        });
    }

    @Test
    void testContains() {
        FList<String> l = FList.of("A", "B", "C");
        assertTrue(l.contains("A"));
        assertTrue(l.contains("B"));
        assertTrue(l.contains("C"));
        assertFalse(l.contains("X"));
        assertFalse(l.contains(null));

        assertFalse(l.tail().contains("A"));
        assertTrue(l.tail().contains("B"));
        assertTrue(l.tail().contains("C"));
        assertFalse(l.tail().contains("X"));
        assertFalse(l.tail().contains(null));

        assertFalse(l.tail().tail().contains("A"));
        assertFalse(l.tail().tail().contains("B"));
        assertTrue(l.tail().tail().contains("C"));
        assertFalse(l.tail().tail().contains("X"));
        assertFalse(l.tail().tail().contains(null));

        assertFalse(l.tail().tail().tail().contains("A"));
        assertFalse(l.tail().tail().tail().contains("B"));
        assertFalse(l.tail().tail().tail().contains("C"));
        assertFalse(l.tail().tail().tail().contains("X"));
        assertFalse(l.tail().tail().tail().contains(null));
    }

    @Test
    void testEquals() {
        FList<String> l = FList.of("A", "B", "C");
        assertTrue(l.equals(FList.of("A", "B", "C")));
        assertTrue(l.tail().equals(FList.of("B", "C")));
        assertTrue(l.tail().tail().equals(FList.of("C")));
        assertTrue(l.tail().tail().tail().equals(FList.of()));
        assertTrue(l.tail().tail().tail().equals(FList.empty()));

        assertFalse(l.equals(FList.of("A", "B")));
        assertFalse(l.equals(FList.of("B", "C", "A")));
        assertFalse(l.tail().equals(FList.of("B")));
        assertFalse(l.tail().tail().tail().equals(FList.of("X")));
    }

    @Test
    void testRemove() {
        FList<String> l = FList.of("A", "B", "C");
        assertEquals("[A, B, C]", l.remove("D").toString());
        assertEquals("[B, C]", l.remove("A").toString());
        assertEquals("[A, C]", l.remove("B").toString());
        assertEquals("[A, B]", l.remove("C").toString());
        assertEquals("[A, B, C]", l.remove("D").toString());
    }

}
