package at.jku.ssw.fp.sect10_8;

import static java.util.concurrent.TimeUnit.MILLISECONDS;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import io.reactivex.Observable;
import io.reactivex.observers.TestObserver;
import io.reactivex.schedulers.TestScheduler;

/**
 * Demo showing the use of {@link TestScheduler} and {@link TestObserver}. 
 * 
 * @author Herbert Praehofer
 */
public class Demo1_ObservableTest {
  Observable<String> obsUnderTest; 
  TestObserver<String> testObserver;  
  TestScheduler testScheduler;

  @BeforeEach
  public void before() {
    testScheduler =  new TestScheduler();
    testObserver = TestObserver.create();
    obsUnderTest = 
      Observable.merge(
        Observable
          .interval(20, 20, MILLISECONDS, testScheduler)
          .map(x -> "F" + x)
          .take(3), 
        Observable
          .timer(50, MILLISECONDS, testScheduler)
          .map(x -> "S" + x));
    obsUnderTest.subscribe(testObserver); 
  }

  @Test
  public void test() {
    testScheduler.advanceTimeBy(10, MILLISECONDS);
    testObserver.assertNoValues();
    testScheduler.advanceTimeTo(20, MILLISECONDS);
    testObserver.assertValue("F0");
    testScheduler.advanceTimeBy(20, MILLISECONDS);
    testObserver.assertValues("F0", "F1");
    testObserver.assertValueCount(2);
    testObserver.assertNotComplete();
    testObserver.assertNoErrors();
    testObserver.assertNotTerminated();
    testScheduler.advanceTimeBy(10, MILLISECONDS);
    testObserver.assertValueAt(2, "S0");
    testScheduler.advanceTimeBy(10, MILLISECONDS);
    testObserver.assertValueCount(4);
    testObserver.assertNever("S1"); 
    testObserver.assertComplete();

	}

}
