package at.jku.ssw.fp.sect11_1;

import static org.junit.jupiter.api.Assertions.assertAll;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTimeout;

import java.time.Duration;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.NoSuchElementException;

import org.junit.jupiter.api.Test;

import at.jku.ssw.fp.airline.Airline;
import at.jku.ssw.fp.coll.FList;

/**
 * Demo showing the new functional features of JUnit 5.
 *
 * @author Herbert Praehofer
 *
 */
class Demo_JUnit5 {

  private LocalDate nextWeek = LocalDate.now().plusDays(7);
  private Airline lufthansa = Airline.AIRLINES[0];

  @Test
  public void testFList() {
    FList<Integer> lst = FList.<Integer>empty().add(1).add(2).add(3);
    assertAll(
        () -> assertEquals(1, lst.head()),
        () -> assertEquals(2, lst.tail().head()),
        () -> assertEquals(3, lst.tail().tail().head())
    );
  }

  @Test
  public void testEmptyList() {
    assertThrows(NoSuchElementException.class, () -> FList.empty().head());
    assertThrows(NoSuchElementException.class, () -> FList.empty().tail());
    assertDoesNotThrow(() -> FList.empty().contains(1));
  }

  @Test
  public void testFindFlightTimeout() {
    assertTimeout(Duration.of(3, ChronoUnit.SECONDS), () -> lufthansa.findFlight("VIE", "JKF", nextWeek));
  }

}
