package at.jku.ssw.fp.sect11_2;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.Test;

import at.jku.ssw.fp.util.Util;

/**
 * Demo showing the new functional features of AssertJ.
 *
 * @author Herbert Praehofer
 */
public class Demo_AssertJ {

  private Map<Integer, String> namesMap = Map.of(1, "Hans", 2, "Fritz");

    @Test
    public void testStreams() {
        List<String> words = List.of("Funktionale", "Programmierung");
        assertThat(words)
          .contains("Funktionale")
          .contains("Programmierung")
          .doesNotContain("Java")
          .containsExactlyInAnyOrder("Funktionale", "Programmierung")
          .extracting(s -> s.length())
          .containsSequence(11, 14);

        Stream<String> stream = Stream.of("Funktionale", "Programmierung");
        assertThat(stream)
                .contains("Funktionale")
                .contains("Programmierung")
                .containsExactlyInAnyOrder("Funktionale", "Programmierung")
                .doesNotContain("Java")
                .flatExtracting(s -> s.chars().distinct().mapToObj(i -> Character.valueOf((char)i)).collect(Collectors.toList()))
                .contains('P', 'r', 'o', 'm')
                .doesNotContain('x', 'M', 'p');

    }

    @Test
    public void testOptional() {
      Optional<String> optString = Util.lookup(namesMap, 1);
          assertThat(optString)
              .isPresent()
              .hasValueSatisfying(s -> assertTrue(s.startsWith("H")))
              .map(s -> s.length())
              .hasValue(4);

    }

    @Test
    public void testPredicate() {

      Predicate<String> pred = s -> s.length() > 10;
      assertThat(pred)
          .accepts("Funktionale", "Programmierung")
          .rejects("in", "Java");

    }

}
